@@ -1,16 +1,20 @@
 use std::io::Cursor;
+use std::sync::Arc;
 
 use criterion::{criterion_group, criterion_main, Criterion};
 
 use arrow2::array::{clone, Array};
+use arrow2::chunk::Chunk;
+use arrow2::datatypes::{Field, Schema};
 use arrow2::error::Result;
 use arrow2::io::parquet::write::*;
-use arrow2::record_batch::RecordBatch;
 use arrow2::util::bench_util::{create_boolean_array, create_primitive_array, create_string_array};
 
+type ChunkArc = Chunk<Arc<dyn Array>>;
+
 fn write(array: &dyn Array, encoding: Encoding) -> Result<()> {
-    let batch = RecordBatch::try_from_iter([("c1", clone(array).into())])?;
-    let schema = batch.schema().clone();
+    let schema = Schema::from(vec![Field::new("c1", array.data_type().clone(), true)]);
+    let columns: ChunkArc = Chunk::new(vec![clone(array).into()]);
 
     let options = WriteOptions {
         write_statistics: false,
@@ -19,7 +23,7 @@ fn write(array: &dyn Array, encoding: Encoding) -> Result<()> {
     };
 
     let row_groups = RowGroupIterator::try_new(
-        vec![Ok(batch)].into_iter(),
+        vec![Ok(columns)].into_iter(),
         &schema,
         options,
         vec![encoding],
