@@ -9,22 +9,25 @@ unsafe impl ToFfi for UnionArray {
     fn buffers(&self) -> Vec<Option<std::ptr::NonNull<u8>>> {
         if let Some(offsets) = &self.offsets {
             vec![
-                None,
-                std::ptr::NonNull::new(self.types.as_ptr() as *mut u8),
-                std::ptr::NonNull::new(offsets.as_ptr() as *mut u8),
+                Some(self.types.as_ptr().cast::<u8>()),
+                Some(offsets.as_ptr().cast::<u8>()),
             ]
         } else {
-            vec![None, std::ptr::NonNull::new(self.types.as_ptr() as *mut u8)]
+            vec![Some(self.types.as_ptr().cast::<u8>())]
         }
     }
 
-    fn offset(&self) -> usize {
-        self.offset
-    }
-
     fn children(&self) -> Vec<Arc<dyn Array>> {
         self.fields.clone()
     }
+
+    fn offset(&self) -> Option<usize> {
+        Some(self.types.offset())
+    }
+
+    fn to_ffi_aligned(&self) -> Self {
+        self.clone()
+    }
 }
 
 impl<A: ffi::ArrowArrayRef> FromFfi<A> for UnionArray {
