@@ -6,20 +6,28 @@ use crate::{
         Array, MutableArray, Offset, TryExtend, TryPush,
     },
     bitmap::MutableBitmap,
-    buffer::MutableBuffer,
     datatypes::DataType,
     error::{ArrowError, Result},
     trusted_len::TrustedLen,
 };
 
 use super::Utf8Array;
+use crate::array::physical_binary::*;
+
+struct Wrapper<P>(P);
+impl<T: AsRef<str>> AsRef<[u8]> for Wrapper<T> {
+    #[inline]
+    fn as_ref(&self) -> &[u8] {
+        self.0.as_ref().as_bytes()
+    }
+}
 
 /// The mutable version of [`Utf8Array`]. See [`MutableArray`] for more details.
 #[derive(Debug)]
 pub struct MutableUtf8Array<O: Offset> {
     data_type: DataType,
-    offsets: MutableBuffer<O>,
-    values: MutableBuffer<u8>,
+    offsets: Vec<O>,
+    values: Vec<u8>,
     validity: Option<MutableBitmap>,
 }
 
@@ -48,12 +56,11 @@ impl<O: Offset> Default for MutableUtf8Array<O> {
 impl<O: Offset> MutableUtf8Array<O> {
     /// Initializes a new empty [`MutableUtf8Array`].
     pub fn new() -> Self {
-        let mut offsets = MutableBuffer::<O>::new();
-        offsets.push(O::default());
+        let offsets = vec![O::default()];
         Self {
             data_type: Self::default_data_type(),
             offsets,
-            values: MutableBuffer::<u8>::new(),
+            values: Vec::<u8>::new(),
             validity: None,
         }
     }
@@ -66,8 +73,8 @@ impl<O: Offset> MutableUtf8Array<O> {
     /// * The validity is not `None` and its length is different from `offsets`'s length minus one.
     pub fn from_data(
         data_type: DataType,
-        offsets: MutableBuffer<O>,
-        values: MutableBuffer<u8>,
+        offsets: Vec<O>,
+        values: Vec<u8>,
         validity: Option<MutableBitmap>,
     ) -> Self {
         check_offsets_and_utf8(&offsets, &values);
@@ -94,8 +101,8 @@ impl<O: Offset> MutableUtf8Array<O> {
     /// * The validity is not `None` and its length is different from `offsets`'s length minus one.
     pub unsafe fn from_data_unchecked(
         data_type: DataType,
-        offsets: MutableBuffer<O>,
-        values: MutableBuffer<u8>,
+        offsets: Vec<O>,
+        values: Vec<u8>,
         validity: Option<MutableBitmap>,
     ) -> Self {
         check_offsets_minimal(&offsets, values.len());
@@ -124,13 +131,13 @@ impl<O: Offset> MutableUtf8Array<O> {
 
     /// Initializes a new [`MutableUtf8Array`] with a pre-allocated capacity of slots and values.
     pub fn with_capacities(capacity: usize, values: usize) -> Self {
-        let mut offsets = MutableBuffer::<O>::with_capacity(capacity + 1);
+        let mut offsets = Vec::<O>::with_capacity(capacity + 1);
         offsets.push(O::default());
 
         Self {
             data_type: Self::default_data_type(),
             offsets,
-            values: MutableBuffer::<u8>::with_capacity(values),
+            values: Vec::<u8>::with_capacity(values),
             validity: None,
         }
     }
@@ -181,12 +188,12 @@ impl<O: Offset> MutableUtf8Array<O> {
 
 impl<O: Offset> MutableUtf8Array<O> {
     /// returns its values.
-    pub fn values(&self) -> &MutableBuffer<u8> {
+    pub fn values(&self) -> &Vec<u8> {
         &self.values
     }
 
     /// returns its offsets.
-    pub fn offsets(&self) -> &MutableBuffer<O> {
+    pub fn offsets(&self) -> &Vec<O> {
         &self.offsets
     }
 }
@@ -201,21 +208,31 @@ impl<O: Offset> MutableArray for MutableUtf8Array<O> {
     }
 
     fn as_box(&mut self) -> Box<dyn Array> {
-        Box::new(Utf8Array::from_data(
-            Self::default_data_type(),
-            std::mem::take(&mut self.offsets).into(),
-            std::mem::take(&mut self.values).into(),
-            std::mem::take(&mut self.validity).map(|x| x.into()),
-        ))
+        // Safety:
+        // `MutableUtf8Array` has the same invariants as `Utf8Array` and thus
+        // `Utf8Array` can be safely created from `MutableUtf8Array` without checks.
+        Box::new(unsafe {
+            Utf8Array::from_data_unchecked(
+                self.data_type.clone(),
+                std::mem::take(&mut self.offsets).into(),
+                std::mem::take(&mut self.values).into(),
+                std::mem::take(&mut self.validity).map(|x| x.into()),
+            )
+        })
     }
 
     fn as_arc(&mut self) -> Arc<dyn Array> {
-        Arc::new(Utf8Array::from_data(
-            Self::default_data_type(),
-            std::mem::take(&mut self.offsets).into(),
-            std::mem::take(&mut self.values).into(),
-            std::mem::take(&mut self.validity).map(|x| x.into()),
-        ))
+        // Safety:
+        // `MutableUtf8Array` has the same invariants as `Utf8Array` and thus
+        // `Utf8Array` can be safely created from `MutableUtf8Array` without checks.
+        Arc::new(unsafe {
+            Utf8Array::from_data_unchecked(
+                self.data_type.clone(),
+                std::mem::take(&mut self.offsets).into(),
+                std::mem::take(&mut self.values).into(),
+                std::mem::take(&mut self.validity).map(|x| x.into()),
+            )
+        })
     }
 
     fn data_type(&self) -> &DataType {
@@ -275,6 +292,7 @@ impl<O: Offset> MutableUtf8Array<O> {
         let (_, upper) = iterator.size_hint();
         let additional = upper.expect("extend_trusted_len_values requires an upper limit");
 
+        let iterator = iterator.map(Wrapper);
         extend_from_trusted_len_values_iter(&mut self.offsets, &mut self.values, iterator);
 
         if let Some(validity) = self.validity.as_mut() {
@@ -307,6 +325,7 @@ impl<O: Offset> MutableUtf8Array<O> {
             self.validity = Some(validity);
         }
 
+        let iterator = iterator.map(|x| x.map(Wrapper));
         extend_from_trusted_len_iter(
             &mut self.offsets,
             &mut self.values,
@@ -329,6 +348,7 @@ impl<O: Offset> MutableUtf8Array<O> {
         P: AsRef<str>,
         I: Iterator<Item = Option<P>>,
     {
+        let iterator = iterator.map(|x| x.map(Wrapper));
         let (validity, offsets, values) = trusted_len_unzip(iterator);
 
         // soundness: P is `str`
@@ -354,12 +374,13 @@ impl<O: Offset> MutableUtf8Array<O> {
     pub unsafe fn from_trusted_len_values_iter_unchecked<T: AsRef<str>, I: Iterator<Item = T>>(
         iterator: I,
     ) -> Self {
+        let iterator = iterator.map(Wrapper);
         let (offsets, values) = unsafe { trusted_len_values_iter(iterator) };
         // soundness: T is AsRef<str>
         Self::from_data_unchecked(Self::default_data_type(), offsets, values, None)
     }
 
-    /// Creates a new [`Utf8Array`] from a [`TrustedLen`] of `&str`.
+    /// Creates a new [`MutableUtf8Array`] from a [`TrustedLen`] of `&str`.
     #[inline]
     pub fn from_trusted_len_values_iter<T: AsRef<str>, I: TrustedLen<Item = T>>(
         iterator: I,
@@ -396,7 +417,7 @@ impl<O: Offset> MutableUtf8Array<O> {
     {
         let iterator = iterator.into_iter();
 
-        // soundness: assumed trusted len
+        let iterator = iterator.map(|x| x.map(|x| x.map(Wrapper)));
         let (validity, offsets, values) = try_trusted_len_unzip(iterator)?;
 
         // soundness: P is `str`
@@ -421,6 +442,7 @@ impl<O: Offset> MutableUtf8Array<O> {
 
     /// Creates a new [`MutableUtf8Array`] from a [`Iterator`] of `&str`.
     pub fn from_iter_values<T: AsRef<str>, I: Iterator<Item = T>>(iterator: I) -> Self {
+        let iterator = iterator.map(Wrapper);
         let (offsets, values) = values_iter(iterator);
         // soundness: T: AsRef<str>
         unsafe { Self::from_data_unchecked(Self::default_data_type(), offsets, values, None) }
@@ -468,231 +490,3 @@ impl<O: Offset, T: AsRef<str>> TryPush<Option<T>> for MutableUtf8Array<O> {
         Ok(())
     }
 }
-
-/// Creates [`MutableBitmap`] and two [`MutableBuffer`]s from an iterator of `Option`.
-/// The first buffer corresponds to a offset buffer, the second one
-/// corresponds to a values buffer.
-/// # Safety
-/// The caller must ensure that `iterator` is `TrustedLen`.
-#[inline]
-unsafe fn trusted_len_unzip<O, I, P>(
-    iterator: I,
-) -> (Option<MutableBitmap>, MutableBuffer<O>, MutableBuffer<u8>)
-where
-    O: Offset,
-    P: AsRef<str>,
-    I: Iterator<Item = Option<P>>,
-{
-    let mut offsets = MutableBuffer::<O>::with_capacity(1);
-    let mut values = MutableBuffer::<u8>::new();
-    let mut validity = MutableBitmap::new();
-
-    offsets.push_unchecked(O::default());
-
-    extend_from_trusted_len_iter(&mut offsets, &mut values, &mut validity, iterator);
-
-    let validity = if validity.null_count() > 0 {
-        Some(validity)
-    } else {
-        None
-    };
-
-    (validity, offsets, values)
-}
-
-/// # Safety
-/// The caller must ensure that `iterator` is `TrustedLen`.
-#[inline]
-#[allow(clippy::type_complexity)]
-pub(crate) unsafe fn try_trusted_len_unzip<E, I, P, O>(
-    iterator: I,
-) -> std::result::Result<(Option<MutableBitmap>, MutableBuffer<O>, MutableBuffer<u8>), E>
-where
-    O: Offset,
-    P: AsRef<str>,
-    I: Iterator<Item = std::result::Result<Option<P>, E>>,
-{
-    let (_, upper) = iterator.size_hint();
-    let len = upper.expect("trusted_len_unzip requires an upper limit");
-
-    let mut validity = MutableBitmap::with_capacity(len);
-    let mut offsets = MutableBuffer::<O>::with_capacity(len + 1);
-    let mut values = MutableBuffer::<u8>::new();
-
-    let mut length = O::default();
-    let mut dst = offsets.as_mut_ptr();
-    std::ptr::write(dst, length);
-    dst = dst.add(1);
-    for item in iterator {
-        if let Some(item) = item? {
-            validity.push(true);
-            let s = item.as_ref();
-            length += O::from_usize(s.len()).unwrap();
-            values.extend_from_slice(s.as_bytes());
-        } else {
-            validity.push(false);
-        };
-        std::ptr::write(dst, length);
-        dst = dst.add(1);
-    }
-    assert_eq!(
-        dst.offset_from(offsets.as_ptr()) as usize,
-        len + 1,
-        "Trusted iterator length was not accurately reported"
-    );
-    offsets.set_len(len + 1);
-
-    let validity = if validity.null_count() > 0 {
-        Some(validity)
-    } else {
-        None
-    };
-
-    Ok((validity, offsets, values))
-}
-
-/// Creates two [`Buffer`]s from an iterator of `&str`.
-/// The first buffer corresponds to a offset buffer, the second to a values buffer.
-/// # Safety
-/// The caller must ensure that `iterator` is [`TrustedLen`].
-#[inline]
-pub(crate) unsafe fn trusted_len_values_iter<O, I, P>(
-    iterator: I,
-) -> (MutableBuffer<O>, MutableBuffer<u8>)
-where
-    O: Offset,
-    P: AsRef<str>,
-    I: Iterator<Item = P>,
-{
-    let mut offsets = MutableBuffer::<O>::with_capacity(1 + iterator.size_hint().1.unwrap());
-    let mut values = MutableBuffer::<u8>::new();
-
-    offsets.push_unchecked(O::default());
-
-    extend_from_trusted_len_values_iter(&mut offsets, &mut values, iterator);
-
-    (offsets, values)
-}
-
-/// Populates `offsets` and `values` [`Buffer`] with information
-/// extracted from the incoming iterator.
-/// # Safety
-/// The caller must ensure that `iterator` is [`TrustedLen`]
-#[inline]
-unsafe fn extend_from_trusted_len_values_iter<I, P, O>(
-    offsets: &mut MutableBuffer<O>,
-    values: &mut MutableBuffer<u8>,
-    iterator: I,
-) where
-    O: Offset,
-    P: AsRef<str>,
-    I: Iterator<Item = P>,
-{
-    let (_, upper) = iterator.size_hint();
-    let additional = upper.expect("extend_from_trusted_len_iter_values requires an upper limit");
-
-    offsets.reserve(additional);
-
-    let mut length = *offsets.last().unwrap();
-
-    let mut dst = offsets.as_mut_ptr();
-    dst = dst.add(offsets.len());
-
-    for item in iterator {
-        let s = item.as_ref();
-
-        length += O::from_usize(s.len()).unwrap();
-
-        values.extend_from_slice(s.as_bytes());
-        std::ptr::write(dst, length);
-
-        dst = dst.add(1);
-    }
-
-    assert_eq!(
-        dst.offset_from(offsets.as_ptr()) as usize,
-        offsets.len() + additional,
-        "Trusted iterator length was not accurately reported"
-    );
-
-    offsets.set_len(offsets.len() + additional);
-}
-
-/// Populates `offsets`, `values`, and validity [`Buffer`] with information
-/// extracted from the incoming iterator.
-/// # Safety
-/// The caller must ensure that `iterator` is [`TrustedLen`]
-#[inline]
-unsafe fn extend_from_trusted_len_iter<O, I, P>(
-    offsets: &mut MutableBuffer<O>,
-    values: &mut MutableBuffer<u8>,
-    validity: &mut MutableBitmap,
-    iterator: I,
-) where
-    O: Offset,
-    P: AsRef<str>,
-    I: Iterator<Item = Option<P>>,
-{
-    let (_, upper) = iterator.size_hint();
-    let additional = upper.expect("extend_from_trusted_len_values_iter requires an upper limit");
-
-    offsets.reserve(additional);
-    validity.reserve(additional);
-
-    let mut length = *offsets.last().unwrap();
-
-    let mut dst = offsets.as_mut_ptr();
-    dst = dst.add(offsets.len());
-
-    for item in iterator {
-        if let Some(item) = item {
-            let s = item.as_ref();
-
-            length += O::from_usize(s.len()).unwrap();
-
-            values.extend_from_slice(s.as_bytes());
-            validity.push_unchecked(true);
-        } else {
-            validity.push_unchecked(false);
-        };
-
-        std::ptr::write(dst, length);
-
-        dst = dst.add(1);
-    }
-
-    assert_eq!(
-        dst.offset_from(offsets.as_ptr()) as usize,
-        offsets.len() + additional,
-        "Trusted iterator length was not accurately reported"
-    );
-
-    offsets.set_len(offsets.len() + additional);
-}
-
-/// Creates two [`MutableBuffer`]s from an iterator of `&str`.
-/// The first buffer corresponds to a offset buffer, the second to a values buffer.
-#[inline]
-fn values_iter<O, I, P>(iterator: I) -> (MutableBuffer<O>, MutableBuffer<u8>)
-where
-    O: Offset,
-    P: AsRef<str>,
-    I: Iterator<Item = P>,
-{
-    let (lower, _) = iterator.size_hint();
-
-    let mut offsets = MutableBuffer::<O>::with_capacity(lower + 1);
-    let mut values = MutableBuffer::<u8>::new();
-
-    let mut length = O::default();
-    offsets.push(length);
-
-    for item in iterator {
-        let s = item.as_ref();
-        length += O::from_usize(s.len()).unwrap();
-        values.extend_from_slice(s.as_bytes());
-
-        offsets.push(length)
-    }
-    (offsets, values)
-}
