@@ -1,24 +1,25 @@
 //! Defines different casting operators such as [`cast`] or [`primitive_to_binary`].
 
-use crate::{
-    array::*,
-    buffer::Buffer,
-    datatypes::*,
-    error::{ArrowError, Result},
-};
-
 mod binary_to;
 mod boolean_to;
+mod decimal_to;
 mod dictionary_to;
 mod primitive_to;
 mod utf8_to;
 
 pub use binary_to::*;
 pub use boolean_to::*;
+pub use decimal_to::*;
 pub use dictionary_to::*;
 pub use primitive_to::*;
 pub use utf8_to::*;
 
+use crate::{
+    array::*,
+    datatypes::*,
+    error::{ArrowError, Result},
+};
+
 /// options defining how Cast kernels behave
 #[derive(Clone, Copy, Debug, Default)]
 pub struct CastOptions {
@@ -80,58 +81,30 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
     match (from_type, to_type) {
         (
             Null,
-            Boolean
-            | Int8
-            | UInt8
-            | Int16
-            | UInt16
-            | Int32
-            | UInt32
-            | Float32
-            | Date32
-            | Time32(_)
-            | Int64
-            | UInt64
-            | Float64
-            | Date64
-            | List(_)
-            | Dictionary(_, _),
+            Boolean | Int8 | UInt8 | Int16 | UInt16 | Int32 | UInt32 | Float32 | Date32 | Time32(_)
+            | Int64 | UInt64 | Float64 | Date64 | List(_) | Dictionary(..),
         )
         | (
-            Boolean
-            | Int8
-            | UInt8
-            | Int16
-            | UInt16
-            | Int32
-            | UInt32
-            | Float32
-            | Date32
-            | Time32(_)
-            | Int64
-            | UInt64
-            | Float64
-            | Date64
-            | List(_)
-            | Dictionary(_, _),
+            Boolean | Int8 | UInt8 | Int16 | UInt16 | Int32 | UInt32 | Float32 | Date32 | Time32(_)
+            | Int64 | UInt64 | Float64 | Date64 | List(_) | Dictionary(..),
             Null,
         ) => true,
         (Struct(_), _) => false,
         (_, Struct(_)) => false,
         (List(list_from), List(list_to)) => {
-            can_cast_types(list_from.data_type(), list_to.data_type())
+            can_cast_types(&list_from.data_type, &list_to.data_type)
         }
         (LargeList(list_from), LargeList(list_to)) => {
-            can_cast_types(list_from.data_type(), list_to.data_type())
+            can_cast_types(&list_from.data_type, &list_to.data_type)
         }
         (List(list_from), LargeList(list_to)) if list_from == list_to => true,
         (LargeList(list_from), List(list_to)) if list_from == list_to => true,
-        (_, List(list_to)) => can_cast_types(from_type, list_to.data_type()),
-        (Dictionary(_, from_value_type), Dictionary(_, to_value_type)) => {
+        (_, List(list_to)) => can_cast_types(from_type, &list_to.data_type),
+        (Dictionary(_, from_value_type, _), Dictionary(_, to_value_type, _)) => {
             can_cast_types(from_value_type, to_value_type)
         }
-        (Dictionary(_, value_type), _) => can_cast_types(value_type, to_type),
-        (_, Dictionary(_, value_type)) => can_cast_types(from_type, value_type),
+        (Dictionary(_, value_type, _), _) => can_cast_types(value_type, to_type),
+        (_, Dictionary(_, value_type, _)) => can_cast_types(from_type, value_type),
 
         (_, Boolean) => is_numeric(from_type),
         (Boolean, _) => {
@@ -172,6 +145,7 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
         (UInt8, Int64) => true,
         (UInt8, Float32) => true,
         (UInt8, Float64) => true,
+        (UInt8, Decimal(_, _)) => true,
 
         (UInt16, UInt8) => true,
         (UInt16, UInt32) => true,
@@ -182,6 +156,7 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
         (UInt16, Int64) => true,
         (UInt16, Float32) => true,
         (UInt16, Float64) => true,
+        (UInt16, Decimal(_, _)) => true,
 
         (UInt32, UInt8) => true,
         (UInt32, UInt16) => true,
@@ -192,6 +167,7 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
         (UInt32, Int64) => true,
         (UInt32, Float32) => true,
         (UInt32, Float64) => true,
+        (UInt32, Decimal(_, _)) => true,
 
         (UInt64, UInt8) => true,
         (UInt64, UInt16) => true,
@@ -202,6 +178,7 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
         (UInt64, Int64) => true,
         (UInt64, Float32) => true,
         (UInt64, Float64) => true,
+        (UInt64, Decimal(_, _)) => true,
 
         (Int8, UInt8) => true,
         (Int8, UInt16) => true,
@@ -212,6 +189,7 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
         (Int8, Int64) => true,
         (Int8, Float32) => true,
         (Int8, Float64) => true,
+        (Int8, Decimal(_, _)) => true,
 
         (Int16, UInt8) => true,
         (Int16, UInt16) => true,
@@ -222,6 +200,7 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
         (Int16, Int64) => true,
         (Int16, Float32) => true,
         (Int16, Float64) => true,
+        (Int16, Decimal(_, _)) => true,
 
         (Int32, UInt8) => true,
         (Int32, UInt16) => true,
@@ -232,6 +211,7 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
         (Int32, Int64) => true,
         (Int32, Float32) => true,
         (Int32, Float64) => true,
+        (Int32, Decimal(_, _)) => true,
 
         (Int64, UInt8) => true,
         (Int64, UInt16) => true,
@@ -242,6 +222,7 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
         (Int64, Int32) => true,
         (Int64, Float32) => true,
         (Int64, Float64) => true,
+        (Int64, Decimal(_, _)) => true,
 
         (Float32, UInt8) => true,
         (Float32, UInt16) => true,
@@ -252,6 +233,7 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
         (Float32, Int32) => true,
         (Float32, Int64) => true,
         (Float32, Float64) => true,
+        (Float32, Decimal(_, _)) => true,
 
         (Float64, UInt8) => true,
         (Float64, UInt16) => true,
@@ -262,6 +244,22 @@ pub fn can_cast_types(from_type: &DataType, to_type: &DataType) -> bool {
         (Float64, Int32) => true,
         (Float64, Int64) => true,
         (Float64, Float32) => true,
+        (Float64, Decimal(_, _)) => true,
+
+        (
+            Decimal(_, _),
+            UInt8
+            | UInt16
+            | UInt32
+            | UInt64
+            | Int8
+            | Int16
+            | Int32
+            | Int64
+            | Float32
+            | Float64
+            | Decimal(_, _),
+        ) => true,
         // end numeric casts
 
         // temporal casts
@@ -318,9 +316,12 @@ fn cast_list<O: Offset>(
 }
 
 fn cast_list_to_large_list(array: &ListArray<i32>, to_type: &DataType) -> ListArray<i64> {
-    let offsets = array.offsets();
-    let offsets = offsets.iter().map(|x| *x as i64);
-    let offets = Buffer::from_trusted_len_iter(offsets);
+    let offets = array
+        .offsets()
+        .iter()
+        .map(|x| *x as i64)
+        .collect::<Vec<_>>()
+        .into();
 
     ListArray::<i64>::from_data(
         to_type.clone(),
@@ -331,13 +332,16 @@ fn cast_list_to_large_list(array: &ListArray<i32>, to_type: &DataType) -> ListAr
 }
 
 fn cast_large_to_list(array: &ListArray<i64>, to_type: &DataType) -> ListArray<i32> {
-    let offsets = array.offsets();
-    let offsets = offsets.iter().map(|x| *x as i32);
-    let offets = Buffer::from_trusted_len_iter(offsets);
+    let offsets = array
+        .offsets()
+        .iter()
+        .map(|x| *x as i32)
+        .collect::<Vec<_>>()
+        .into();
 
     ListArray::<i32>::from_data(
         to_type.clone(),
-        offets,
+        offsets,
         array.values().clone(),
         array.validity().cloned(),
     )
@@ -376,40 +380,12 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
     match (from_type, to_type) {
         (
             Null,
-            Boolean
-            | Int8
-            | UInt8
-            | Int16
-            | UInt16
-            | Int32
-            | UInt32
-            | Float32
-            | Date32
-            | Time32(_)
-            | Int64
-            | UInt64
-            | Float64
-            | Date64
-            | List(_)
-            | Dictionary(_, _),
+            Boolean | Int8 | UInt8 | Int16 | UInt16 | Int32 | UInt32 | Float32 | Date32 | Time32(_)
+            | Int64 | UInt64 | Float64 | Date64 | List(_) | Dictionary(..),
         )
         | (
-            Boolean
-            | Int8
-            | UInt8
-            | Int16
-            | UInt16
-            | Int32
-            | UInt32
-            | Float32
-            | Date32
-            | Time32(_)
-            | Int64
-            | UInt64
-            | Float64
-            | Date64
-            | List(_)
-            | Dictionary(_, _),
+            Boolean | Int8 | UInt8 | Int16 | UInt16 | Int32 | UInt32 | Float32 | Date32 | Time32(_)
+            | Int64 | UInt64 | Float64 | Date64 | List(_) | Dictionary(..),
             Null,
         ) => Ok(new_null_array(to_type.clone(), array.len())),
         (Struct(_), _) => Err(ArrowError::NotYetImplemented(
@@ -439,20 +415,20 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
 
         (_, List(to)) => {
             // cast primitive to list's primitive
-            let values = cast(array, to.data_type(), options)?.into();
+            let values = cast(array, &to.data_type, options)?.into();
             // create offsets, where if array.len() = 2, we have [0,1,2]
-            let offsets =
-                unsafe { Buffer::from_trusted_len_iter_unchecked(0..=array.len() as i32) };
+            let offsets = (0..=array.len() as i32).collect::<Vec<_>>();
 
-            let list_array = ListArray::<i32>::from_data(to_type.clone(), offsets, values, None);
+            let list_array =
+                ListArray::<i32>::from_data(to_type.clone(), offsets.into(), values, None);
 
             Ok(Box::new(list_array))
         }
 
-        (Dictionary(index_type, _), _) => match_integer_type!(index_type, |$T| {
+        (Dictionary(index_type, ..), _) => match_integer_type!(index_type, |$T| {
             dictionary_cast_dyn::<$T>(array, to_type, options)
         }),
-        (_, Dictionary(index_type, value_type)) => match_integer_type!(index_type, |$T| {
+        (_, Dictionary(index_type, value_type, _)) => match_integer_type!(index_type, |$T| {
             cast_to_dictionary::<$T>(array, value_type, options)
         }),
         (_, Boolean) => match from_type {
@@ -700,6 +676,7 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
         (UInt8, Int64) => primitive_to_primitive_dyn::<u8, i64>(array, to_type, options),
         (UInt8, Float32) => primitive_to_primitive_dyn::<u8, f32>(array, to_type, as_options),
         (UInt8, Float64) => primitive_to_primitive_dyn::<u8, f64>(array, to_type, as_options),
+        (UInt8, Decimal(p, s)) => integer_to_decimal_dyn::<u8>(array, *p, *s),
 
         (UInt16, UInt8) => primitive_to_primitive_dyn::<u16, u8>(array, to_type, options),
         (UInt16, UInt32) => primitive_to_primitive_dyn::<u16, u32>(array, to_type, as_options),
@@ -710,6 +687,7 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
         (UInt16, Int64) => primitive_to_primitive_dyn::<u16, i64>(array, to_type, options),
         (UInt16, Float32) => primitive_to_primitive_dyn::<u16, f32>(array, to_type, as_options),
         (UInt16, Float64) => primitive_to_primitive_dyn::<u16, f64>(array, to_type, as_options),
+        (UInt16, Decimal(p, s)) => integer_to_decimal_dyn::<u16>(array, *p, *s),
 
         (UInt32, UInt8) => primitive_to_primitive_dyn::<u32, u8>(array, to_type, options),
         (UInt32, UInt16) => primitive_to_primitive_dyn::<u32, u16>(array, to_type, options),
@@ -720,6 +698,7 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
         (UInt32, Int64) => primitive_to_primitive_dyn::<u32, i64>(array, to_type, options),
         (UInt32, Float32) => primitive_to_primitive_dyn::<u32, f32>(array, to_type, as_options),
         (UInt32, Float64) => primitive_to_primitive_dyn::<u32, f64>(array, to_type, as_options),
+        (UInt32, Decimal(p, s)) => integer_to_decimal_dyn::<u32>(array, *p, *s),
 
         (UInt64, UInt8) => primitive_to_primitive_dyn::<u64, u8>(array, to_type, options),
         (UInt64, UInt16) => primitive_to_primitive_dyn::<u64, u16>(array, to_type, options),
@@ -730,6 +709,7 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
         (UInt64, Int64) => primitive_to_primitive_dyn::<u64, i64>(array, to_type, options),
         (UInt64, Float32) => primitive_to_primitive_dyn::<u64, f32>(array, to_type, as_options),
         (UInt64, Float64) => primitive_to_primitive_dyn::<u64, f64>(array, to_type, as_options),
+        (UInt64, Decimal(p, s)) => integer_to_decimal_dyn::<u64>(array, *p, *s),
 
         (Int8, UInt8) => primitive_to_primitive_dyn::<i8, u8>(array, to_type, options),
         (Int8, UInt16) => primitive_to_primitive_dyn::<i8, u16>(array, to_type, options),
@@ -740,6 +720,7 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
         (Int8, Int64) => primitive_to_primitive_dyn::<i8, i64>(array, to_type, as_options),
         (Int8, Float32) => primitive_to_primitive_dyn::<i8, f32>(array, to_type, as_options),
         (Int8, Float64) => primitive_to_primitive_dyn::<i8, f64>(array, to_type, as_options),
+        (Int8, Decimal(p, s)) => integer_to_decimal_dyn::<i8>(array, *p, *s),
 
         (Int16, UInt8) => primitive_to_primitive_dyn::<i16, u8>(array, to_type, options),
         (Int16, UInt16) => primitive_to_primitive_dyn::<i16, u16>(array, to_type, options),
@@ -750,6 +731,7 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
         (Int16, Int64) => primitive_to_primitive_dyn::<i16, i64>(array, to_type, as_options),
         (Int16, Float32) => primitive_to_primitive_dyn::<i16, f32>(array, to_type, as_options),
         (Int16, Float64) => primitive_to_primitive_dyn::<i16, f64>(array, to_type, as_options),
+        (Int16, Decimal(p, s)) => integer_to_decimal_dyn::<i16>(array, *p, *s),
 
         (Int32, UInt8) => primitive_to_primitive_dyn::<i32, u8>(array, to_type, options),
         (Int32, UInt16) => primitive_to_primitive_dyn::<i32, u16>(array, to_type, options),
@@ -760,6 +742,7 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
         (Int32, Int64) => primitive_to_primitive_dyn::<i32, i64>(array, to_type, as_options),
         (Int32, Float32) => primitive_to_primitive_dyn::<i32, f32>(array, to_type, as_options),
         (Int32, Float64) => primitive_to_primitive_dyn::<i32, f64>(array, to_type, as_options),
+        (Int32, Decimal(p, s)) => integer_to_decimal_dyn::<i32>(array, *p, *s),
 
         (Int64, UInt8) => primitive_to_primitive_dyn::<i64, u8>(array, to_type, options),
         (Int64, UInt16) => primitive_to_primitive_dyn::<i64, u16>(array, to_type, options),
@@ -770,6 +753,7 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
         (Int64, Int32) => primitive_to_primitive_dyn::<i64, i32>(array, to_type, options),
         (Int64, Float32) => primitive_to_primitive_dyn::<i64, f32>(array, to_type, options),
         (Int64, Float64) => primitive_to_primitive_dyn::<i64, f64>(array, to_type, as_options),
+        (Int64, Decimal(p, s)) => integer_to_decimal_dyn::<i64>(array, *p, *s),
 
         (Float32, UInt8) => primitive_to_primitive_dyn::<f32, u8>(array, to_type, options),
         (Float32, UInt16) => primitive_to_primitive_dyn::<f32, u16>(array, to_type, options),
@@ -780,6 +764,7 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
         (Float32, Int32) => primitive_to_primitive_dyn::<f32, i32>(array, to_type, options),
         (Float32, Int64) => primitive_to_primitive_dyn::<f32, i64>(array, to_type, options),
         (Float32, Float64) => primitive_to_primitive_dyn::<f32, f64>(array, to_type, as_options),
+        (Float32, Decimal(p, s)) => float_to_decimal_dyn::<f32>(array, *p, *s),
 
         (Float64, UInt8) => primitive_to_primitive_dyn::<f64, u8>(array, to_type, options),
         (Float64, UInt16) => primitive_to_primitive_dyn::<f64, u16>(array, to_type, options),
@@ -790,6 +775,19 @@ pub fn cast(array: &dyn Array, to_type: &DataType, options: CastOptions) -> Resu
         (Float64, Int32) => primitive_to_primitive_dyn::<f64, i32>(array, to_type, options),
         (Float64, Int64) => primitive_to_primitive_dyn::<f64, i64>(array, to_type, options),
         (Float64, Float32) => primitive_to_primitive_dyn::<f64, f32>(array, to_type, options),
+        (Float64, Decimal(p, s)) => float_to_decimal_dyn::<f64>(array, *p, *s),
+
+        (Decimal(_, _), UInt8) => decimal_to_integer_dyn::<u8>(array),
+        (Decimal(_, _), UInt16) => decimal_to_integer_dyn::<u16>(array),
+        (Decimal(_, _), UInt32) => decimal_to_integer_dyn::<u32>(array),
+        (Decimal(_, _), UInt64) => decimal_to_integer_dyn::<u64>(array),
+        (Decimal(_, _), Int8) => decimal_to_integer_dyn::<i8>(array),
+        (Decimal(_, _), Int16) => decimal_to_integer_dyn::<i16>(array),
+        (Decimal(_, _), Int32) => decimal_to_integer_dyn::<i32>(array),
+        (Decimal(_, _), Int64) => decimal_to_integer_dyn::<i64>(array),
+        (Decimal(_, _), Float32) => decimal_to_float_dyn::<f32>(array),
+        (Decimal(_, _), Float64) => decimal_to_float_dyn::<f64>(array),
+        (Decimal(_, _), Decimal(to_p, to_s)) => decimal_to_decimal_dyn(array, *to_p, *to_s),
         // end numeric casts
 
         // temporal casts
