@@ -1,5 +1,8 @@
 use std::hash::Hash;
 
+use num_traits::{AsPrimitive, Float};
+
+use crate::error::Result;
 use crate::{
     array::*,
     bitmap::Bitmap,
@@ -8,10 +11,6 @@ use crate::{
     temporal_conversions::*,
     types::NativeType,
 };
-use crate::{
-    error::Result,
-    util::{lexical_to_bytes, lexical_to_string},
-};
 
 use super::CastOptions;
 
@@ -19,9 +18,34 @@ use super::CastOptions;
 pub fn primitive_to_binary<T: NativeType + lexical_core::ToLexical, O: Offset>(
     from: &PrimitiveArray<T>,
 ) -> BinaryArray<O> {
-    let iter = from.iter().map(|x| x.map(|x| lexical_to_bytes(*x)));
+    let mut values: Vec<u8> = Vec::with_capacity(from.len());
+    let mut offsets: Vec<O> = Vec::with_capacity(from.len() + 1);
+    offsets.push(O::default());
 
-    BinaryArray::from_trusted_len_iter(iter)
+    let mut offset: usize = 0;
+
+    unsafe {
+        for x in from.values().iter() {
+            values.reserve(offset + T::FORMATTED_SIZE_DECIMAL);
+
+            let bytes = std::slice::from_raw_parts_mut(
+                values.as_mut_ptr().add(offset),
+                values.capacity() - offset,
+            );
+            let len = lexical_core::write_unchecked(*x, bytes).len();
+
+            offset += len;
+            offsets.push(O::from_usize(offset as usize).unwrap());
+        }
+        values.set_len(offset);
+        values.shrink_to_fit();
+        BinaryArray::<O>::from_data_unchecked(
+            BinaryArray::<O>::default_data_type(),
+            offsets.into(),
+            values.into(),
+            from.validity().cloned(),
+        )
+    }
 }
 
 pub(super) fn primitive_to_binary_dyn<T, O>(from: &dyn Array) -> Result<Box<dyn Array>>
@@ -60,9 +84,34 @@ where
 pub fn primitive_to_utf8<T: NativeType + lexical_core::ToLexical, O: Offset>(
     from: &PrimitiveArray<T>,
 ) -> Utf8Array<O> {
-    let iter = from.iter().map(|x| x.map(|x| lexical_to_string(*x)));
+    let mut values: Vec<u8> = Vec::with_capacity(from.len());
+    let mut offsets: Vec<O> = Vec::with_capacity(from.len() + 1);
+    offsets.push(O::default());
 
-    Utf8Array::from_trusted_len_iter(iter)
+    let mut offset: usize = 0;
+
+    unsafe {
+        for x in from.values().iter() {
+            values.reserve(offset + T::FORMATTED_SIZE_DECIMAL);
+
+            let bytes = std::slice::from_raw_parts_mut(
+                values.as_mut_ptr().add(offset),
+                values.capacity() - offset,
+            );
+            let len = lexical_core::write_unchecked(*x, bytes).len();
+
+            offset += len;
+            offsets.push(O::from_usize(offset as usize).unwrap());
+        }
+        values.set_len(offset);
+        values.shrink_to_fit();
+        Utf8Array::<O>::from_data_unchecked(
+            Utf8Array::<O>::default_data_type(),
+            offsets.into(),
+            values.into(),
+            from.validity().cloned(),
+        )
+    }
 }
 
 pub(super) fn primitive_to_utf8_dyn<T, O>(from: &dyn Array) -> Result<Box<dyn Array>>
@@ -106,6 +155,94 @@ where
     PrimitiveArray::<O>::from_trusted_len_iter(iter).to(to_type.clone())
 }
 
+/// Returns a [`PrimitiveArray<i128>`] with the casted values. Values are `None` on overflow
+pub fn integer_to_decimal<T: NativeType + AsPrimitive<i128>>(
+    from: &PrimitiveArray<T>,
+    to_precision: usize,
+    to_scale: usize,
+) -> PrimitiveArray<i128> {
+    let multiplier = 10_i128.pow(to_scale as u32);
+
+    let min_for_precision = 9_i128
+        .saturating_pow(1 + to_precision as u32)
+        .saturating_neg();
+    let max_for_precision = 9_i128.saturating_pow(1 + to_precision as u32);
+
+    let values = from.iter().map(|x| {
+        x.and_then(|x| {
+            x.as_().checked_mul(multiplier).and_then(|x| {
+                if x > max_for_precision || x < min_for_precision {
+                    None
+                } else {
+                    Some(x)
+                }
+            })
+        })
+    });
+
+    PrimitiveArray::<i128>::from_trusted_len_iter(values)
+        .to(DataType::Decimal(to_precision, to_scale))
+}
+
+pub(super) fn integer_to_decimal_dyn<T>(
+    from: &dyn Array,
+    precision: usize,
+    scale: usize,
+) -> Result<Box<dyn Array>>
+where
+    T: NativeType + AsPrimitive<i128>,
+{
+    let from = from.as_any().downcast_ref().unwrap();
+    Ok(Box::new(integer_to_decimal::<T>(from, precision, scale)))
+}
+
+/// Returns a [`PrimitiveArray<i128>`] with the casted values. Values are `None` on overflow
+pub fn float_to_decimal<T>(
+    from: &PrimitiveArray<T>,
+    to_precision: usize,
+    to_scale: usize,
+) -> PrimitiveArray<i128>
+where
+    T: NativeType + Float,
+    f64: AsPrimitive<T>,
+    i128: From<T>,
+    T: AsPrimitive<f64>,
+{
+    // 1.2 => 12
+    let multiplier: T = (10_f64).powi(to_scale as i32).as_();
+
+    let min_for_precision = 9_i128
+        .saturating_pow(1 + to_precision as u32)
+        .saturating_neg();
+    let max_for_precision = 9_i128.saturating_pow(1 + to_precision as u32);
+
+    let values = from.iter().map(|x| {
+        x.and_then(|x| {
+            let x = i128::from(*x * multiplier);
+            if x > max_for_precision || x < min_for_precision {
+                None
+            } else {
+                Some(x)
+            }
+        })
+    });
+
+    PrimitiveArray::<i128>::from_trusted_len_iter(values)
+        .to(DataType::Decimal(to_precision, to_scale))
+}
+
+pub(super) fn float_to_decimal_dyn<T>(
+    from: &dyn Array,
+    precision: usize,
+    scale: usize,
+) -> Result<Box<dyn Array>>
+where
+    T: NativeType + AsPrimitive<i128>,
+{
+    let from = from.as_any().downcast_ref().unwrap();
+    Ok(Box::new(integer_to_decimal::<T>(from, precision, scale)))
+}
+
 /// Cast [`PrimitiveArray`] as a [`PrimitiveArray`]
 /// Same as `number as to_number_type` in rust
 pub fn primitive_as_primitive<I, O>(
