@@ -1,7 +1,7 @@
-//! Basic comparison kernels.
+//! Contains comparison operators
 //!
-//! The module contains functions that compare either an array and a scalar
-//! or two arrays of the same [`DataType`]. The scalar-oriented functions are
+//! The module contains functions that compare either an [`Array`] and a [`Scalar`]
+//! or two [`Array`]s (of the same [`DataType`]). The scalar-oriented functions are
 //! suffixed with `_scalar`.
 //!
 //! The functions are organized in two variants:
@@ -54,12 +54,60 @@ pub mod primitive;
 pub mod utf8;
 
 mod simd;
-pub use simd::{Simd8, Simd8Lanes};
+pub use simd::{Simd8, Simd8Lanes, Simd8PartialEq, Simd8PartialOrd};
 
-pub(crate) use primitive::compare_values_op as primitive_compare_values_op;
+pub(crate) use primitive::{
+    compare_values_op as primitive_compare_values_op,
+    compare_values_op_scalar as primitive_compare_values_op_scalar,
+};
+
+macro_rules! match_eq_ord {(
+    $key_type:expr, | $_:tt $T:ident | $($body:tt)*
+) => ({
+    macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
+    use crate::datatypes::PrimitiveType::*;
+    match $key_type {
+        Int8 => __with_ty__! { i8 },
+        Int16 => __with_ty__! { i16 },
+        Int32 => __with_ty__! { i32 },
+        Int64 => __with_ty__! { i64 },
+        Int128 => __with_ty__! { i128 },
+        DaysMs => todo!(),
+        MonthDayNano => todo!(),
+        UInt8 => __with_ty__! { u8 },
+        UInt16 => __with_ty__! { u16 },
+        UInt32 => __with_ty__! { u32 },
+        UInt64 => __with_ty__! { u64 },
+        Float32 => __with_ty__! { f32 },
+        Float64 => __with_ty__! { f64 },
+    }
+})}
+
+macro_rules! match_eq {(
+    $key_type:expr, | $_:tt $T:ident | $($body:tt)*
+) => ({
+    macro_rules! __with_ty__ {( $_ $T:ident ) => ( $($body)* )}
+    use crate::datatypes::PrimitiveType::*;
+    use crate::types::{days_ms, months_days_ns};
+    match $key_type {
+        Int8 => __with_ty__! { i8 },
+        Int16 => __with_ty__! { i16 },
+        Int32 => __with_ty__! { i32 },
+        Int64 => __with_ty__! { i64 },
+        Int128 => __with_ty__! { i128 },
+        DaysMs => __with_ty__! { days_ms },
+        MonthDayNano => __with_ty__! { months_days_ns },
+        UInt8 => __with_ty__! { u8 },
+        UInt16 => __with_ty__! { u16 },
+        UInt32 => __with_ty__! { u32 },
+        UInt64 => __with_ty__! { u64 },
+        Float32 => __with_ty__! { f32 },
+        Float64 => __with_ty__! { f64 },
+    }
+})}
 
 macro_rules! compare {
-    ($lhs:expr, $rhs:expr, $op:tt) => {{
+    ($lhs:expr, $rhs:expr, $op:tt, $p:tt) => {{
         let lhs = $lhs;
         let rhs = $rhs;
         assert_eq!(
@@ -67,65 +115,18 @@ macro_rules! compare {
             rhs.data_type().to_logical_type()
         );
 
-        use DataType::*;
-        let data_type = lhs.data_type().to_logical_type();
-        match data_type {
+        use crate::datatypes::PhysicalType::*;
+        match lhs.data_type().to_physical_type() {
             Boolean => {
                 let lhs = lhs.as_any().downcast_ref().unwrap();
                 let rhs = rhs.as_any().downcast_ref().unwrap();
                 boolean::$op(lhs, rhs)
             }
-            Int8 => {
+            Primitive(primitive) => $p!(primitive, |$T| {
                 let lhs = lhs.as_any().downcast_ref().unwrap();
                 let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<i8>(lhs, rhs)
-            }
-            Int16 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<i16>(lhs, rhs)
-            }
-            Int32 | Date32 | Time32(_) | Interval(IntervalUnit::YearMonth) => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<i32>(lhs, rhs)
-            }
-            Int64 | Timestamp(_, _) | Date64 | Time64(_) | Duration(_) => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<i64>(lhs, rhs)
-            }
-            UInt8 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<u8>(lhs, rhs)
-            }
-            UInt16 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<u16>(lhs, rhs)
-            }
-            UInt32 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<u32>(lhs, rhs)
-            }
-            UInt64 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<u64>(lhs, rhs)
-            }
-            Float16 => unreachable!(),
-            Float32 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<f32>(lhs, rhs)
-            }
-            Float64 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<f64>(lhs, rhs)
-            }
+                primitive::$op::<$T>(lhs, rhs)
+            }),
             Utf8 => {
                 let lhs = lhs.as_any().downcast_ref().unwrap();
                 let rhs = rhs.as_any().downcast_ref().unwrap();
@@ -136,11 +137,6 @@ macro_rules! compare {
                 let rhs = rhs.as_any().downcast_ref().unwrap();
                 utf8::$op::<i64>(lhs, rhs)
             }
-            Decimal(_, _) => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref().unwrap();
-                primitive::$op::<i128>(lhs, rhs)
-            }
             Binary => {
                 let lhs = lhs.as_any().downcast_ref().unwrap();
                 let rhs = rhs.as_any().downcast_ref().unwrap();
@@ -151,7 +147,10 @@ macro_rules! compare {
                 let rhs = rhs.as_any().downcast_ref().unwrap();
                 binary::$op::<i64>(lhs, rhs)
             }
-            _ => todo!("Comparisons of {:?} are not yet supported", data_type),
+            _ => todo!(
+                "Comparison between {:?} are not yet supported",
+                lhs.data_type()
+            ),
         }
     }};
 }
@@ -164,7 +163,7 @@ macro_rules! compare {
 /// * the arrays do not have the same length
 /// * the operation is not supported for the logical type
 pub fn eq(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
-    compare!(lhs, rhs, eq)
+    compare!(lhs, rhs, eq, match_eq)
 }
 
 /// `!=` between two [`Array`]s.
@@ -175,7 +174,7 @@ pub fn eq(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
 /// * the arrays do not have the same length
 /// * the operation is not supported for the logical type
 pub fn neq(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
-    compare!(lhs, rhs, neq)
+    compare!(lhs, rhs, neq, match_eq)
 }
 
 /// `<` between two [`Array`]s.
@@ -186,7 +185,7 @@ pub fn neq(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
 /// * the arrays do not have the same length
 /// * the operation is not supported for the logical type
 pub fn lt(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
-    compare!(lhs, rhs, lt)
+    compare!(lhs, rhs, lt, match_eq_ord)
 }
 
 /// `<=` between two [`Array`]s.
@@ -197,7 +196,7 @@ pub fn lt(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
 /// * the arrays do not have the same length
 /// * the operation is not supported for the logical type
 pub fn lt_eq(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
-    compare!(lhs, rhs, lt_eq)
+    compare!(lhs, rhs, lt_eq, match_eq_ord)
 }
 
 /// `>` between two [`Array`]s.
@@ -208,7 +207,7 @@ pub fn lt_eq(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
 /// * the arrays do not have the same length
 /// * the operation is not supported for the logical type
 pub fn gt(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
-    compare!(lhs, rhs, gt)
+    compare!(lhs, rhs, gt, match_eq_ord)
 }
 
 /// `>=` between two [`Array`]s.
@@ -219,11 +218,11 @@ pub fn gt(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
 /// * the arrays do not have the same length
 /// * the operation is not supported for the logical type
 pub fn gt_eq(lhs: &dyn Array, rhs: &dyn Array) -> BooleanArray {
-    compare!(lhs, rhs, gt_eq)
+    compare!(lhs, rhs, gt_eq, match_eq_ord)
 }
 
 macro_rules! compare_scalar {
-    ($lhs:expr, $rhs:expr, $op:tt) => {{
+    ($lhs:expr, $rhs:expr, $op:tt, $p:tt) => {{
         let lhs = $lhs;
         let rhs = $rhs;
         assert_eq!(
@@ -234,94 +233,40 @@ macro_rules! compare_scalar {
             return BooleanArray::new_null(DataType::Boolean, lhs.len());
         }
 
-        use DataType::*;
-        let data_type = lhs.data_type().to_logical_type();
-        match data_type {
+        use crate::datatypes::PhysicalType::*;
+        match lhs.data_type().to_physical_type() {
             Boolean => {
                 let lhs = lhs.as_any().downcast_ref().unwrap();
                 let rhs = rhs.as_any().downcast_ref::<BooleanScalar>().unwrap();
-                boolean::$op(lhs, rhs.value())
+                // validity checked above
+                boolean::$op(lhs, rhs.value().unwrap())
             }
-            Int8 => {
+            Primitive(primitive) => $p!(primitive, |$T| {
                 let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<i8>>().unwrap();
-                primitive::$op::<i8>(lhs, rhs.value())
-            }
-            Int16 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<i16>>().unwrap();
-                primitive::$op::<i16>(lhs, rhs.value())
-            }
-            Int32 | Date32 | Time32(_) | Interval(IntervalUnit::YearMonth) => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<i32>>().unwrap();
-                primitive::$op::<i32>(lhs, rhs.value())
-            }
-            Int64 | Timestamp(_, _) | Date64 | Time64(_) | Duration(_) => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<i64>>().unwrap();
-                primitive::$op::<i64>(lhs, rhs.value())
-            }
-            UInt8 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<u8>>().unwrap();
-                primitive::$op::<u8>(lhs, rhs.value())
-            }
-            UInt16 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<u16>>().unwrap();
-                primitive::$op::<u16>(lhs, rhs.value())
-            }
-            UInt32 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<u32>>().unwrap();
-                primitive::$op::<u32>(lhs, rhs.value())
-            }
-            UInt64 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<u64>>().unwrap();
-                primitive::$op::<u64>(lhs, rhs.value())
-            }
-            Float16 => unreachable!(),
-            Float32 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<f32>>().unwrap();
-                primitive::$op::<f32>(lhs, rhs.value())
-            }
-            Float64 => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<f64>>().unwrap();
-                primitive::$op::<f64>(lhs, rhs.value())
-            }
+                let rhs = rhs.as_any().downcast_ref::<PrimitiveScalar<$T>>().unwrap();
+                primitive::$op::<$T>(lhs, rhs.value().unwrap())
+            }),
             Utf8 => {
                 let lhs = lhs.as_any().downcast_ref().unwrap();
                 let rhs = rhs.as_any().downcast_ref::<Utf8Scalar<i32>>().unwrap();
-                utf8::$op::<i32>(lhs, rhs.value())
+                utf8::$op::<i32>(lhs, rhs.value().unwrap())
             }
             LargeUtf8 => {
                 let lhs = lhs.as_any().downcast_ref().unwrap();
                 let rhs = rhs.as_any().downcast_ref::<Utf8Scalar<i64>>().unwrap();
-                utf8::$op::<i64>(lhs, rhs.value())
-            }
-            Decimal(_, _) => {
-                let lhs = lhs.as_any().downcast_ref().unwrap();
-                let rhs = rhs
-                    .as_any()
-                    .downcast_ref::<PrimitiveScalar<i128>>()
-                    .unwrap();
-                primitive::$op::<i128>(lhs, rhs.value())
+                utf8::$op::<i64>(lhs, rhs.value().unwrap())
             }
             Binary => {
                 let lhs = lhs.as_any().downcast_ref().unwrap();
                 let rhs = rhs.as_any().downcast_ref::<BinaryScalar<i32>>().unwrap();
-                binary::$op::<i32>(lhs, rhs.value())
+                binary::$op::<i32>(lhs, rhs.value().unwrap())
             }
             LargeBinary => {
                 let lhs = lhs.as_any().downcast_ref().unwrap();
                 let rhs = rhs.as_any().downcast_ref::<BinaryScalar<i64>>().unwrap();
-                binary::$op::<i64>(lhs, rhs.value())
+                binary::$op::<i64>(lhs, rhs.value().unwrap())
             }
-            _ => todo!("Comparisons of {:?} are not yet supported", data_type),
+            _ => todo!("Comparisons of {:?} are not yet supported", lhs.data_type()),
         }
     }};
 }
@@ -333,7 +278,7 @@ macro_rules! compare_scalar {
 /// * they do not have have the same logical type
 /// * the operation is not supported for the logical type
 pub fn eq_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
-    compare_scalar!(lhs, rhs, eq_scalar)
+    compare_scalar!(lhs, rhs, eq_scalar, match_eq)
 }
 
 /// `!=` between an [`Array`] and a [`Scalar`].
@@ -343,7 +288,7 @@ pub fn eq_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
 /// * they do not have have the same logical type
 /// * the operation is not supported for the logical type
 pub fn neq_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
-    compare_scalar!(lhs, rhs, neq_scalar)
+    compare_scalar!(lhs, rhs, neq_scalar, match_eq)
 }
 
 /// `<` between an [`Array`] and a [`Scalar`].
@@ -353,7 +298,7 @@ pub fn neq_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
 /// * they do not have have the same logical type
 /// * the operation is not supported for the logical type
 pub fn lt_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
-    compare_scalar!(lhs, rhs, lt_scalar)
+    compare_scalar!(lhs, rhs, lt_scalar, match_eq_ord)
 }
 
 /// `<=` between an [`Array`] and a [`Scalar`].
@@ -363,7 +308,7 @@ pub fn lt_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
 /// * they do not have have the same logical type
 /// * the operation is not supported for the logical type
 pub fn lt_eq_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
-    compare_scalar!(lhs, rhs, lt_eq_scalar)
+    compare_scalar!(lhs, rhs, lt_eq_scalar, match_eq_ord)
 }
 
 /// `>` between an [`Array`] and a [`Scalar`].
@@ -373,7 +318,7 @@ pub fn lt_eq_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
 /// * they do not have have the same logical type
 /// * the operation is not supported for the logical type
 pub fn gt_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
-    compare_scalar!(lhs, rhs, gt_scalar)
+    compare_scalar!(lhs, rhs, gt_scalar, match_eq_ord)
 }
 
 /// `>=` between an [`Array`] and a [`Scalar`].
@@ -383,41 +328,41 @@ pub fn gt_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
 /// * they do not have have the same logical type
 /// * the operation is not supported for the logical type
 pub fn gt_eq_scalar(lhs: &dyn Array, rhs: &dyn Scalar) -> BooleanArray {
-    compare_scalar!(lhs, rhs, gt_eq_scalar)
+    compare_scalar!(lhs, rhs, gt_eq_scalar, match_eq_ord)
 }
 
-/// Returns whether a [`DataType`] is comparable (either array or scalar) comparison.
+/// Returns whether a [`DataType`] is comparable (either array or scalar).
 pub fn can_eq(data_type: &DataType) -> bool {
-    can_compare(data_type)
+    can_partial_eq(data_type)
 }
 
-/// Returns whether a [`DataType`] is comparable (either array or scalar) comparison.
+/// Returns whether a [`DataType`] is comparable (either array or scalar).
 pub fn can_neq(data_type: &DataType) -> bool {
-    can_compare(data_type)
+    can_partial_eq(data_type)
 }
 
-/// Returns whether a [`DataType`] is comparable (either array or scalar) comparison.
+/// Returns whether a [`DataType`] is comparable (either array or scalar).
 pub fn can_lt(data_type: &DataType) -> bool {
-    can_compare(data_type)
+    can_partial_eq_and_ord(data_type)
 }
 
-/// Returns whether a [`DataType`] is comparable (either array or scalar) comparison.
+/// Returns whether a [`DataType`] is comparable (either array or scalar).
 pub fn can_lt_eq(data_type: &DataType) -> bool {
-    can_compare(data_type)
+    can_partial_eq_and_ord(data_type)
 }
 
-/// Returns whether a [`DataType`] is comparable (either array or scalar) comparison.
+/// Returns whether a [`DataType`] is comparable (either array or scalar).
 pub fn can_gt(data_type: &DataType) -> bool {
-    can_compare(data_type)
+    can_partial_eq_and_ord(data_type)
 }
 
-/// Returns whether a [`DataType`] is comparable (either array or scalar) comparison.
+/// Returns whether a [`DataType`] is comparable (either array or scalar).
 pub fn can_gt_eq(data_type: &DataType) -> bool {
-    can_compare(data_type)
+    can_partial_eq_and_ord(data_type)
 }
 
 // The list of operations currently supported.
-fn can_compare(data_type: &DataType) -> bool {
+fn can_partial_eq_and_ord(data_type: &DataType) -> bool {
     matches!(
         data_type,
         DataType::Boolean
@@ -426,7 +371,7 @@ fn can_compare(data_type: &DataType) -> bool {
             | DataType::Int32
             | DataType::Date32
             | DataType::Time32(_)
-            | DataType::Interval(_)
+            | DataType::Interval(IntervalUnit::YearMonth)
             | DataType::Int64
             | DataType::Timestamp(_, _)
             | DataType::Date64
@@ -445,3 +390,13 @@ fn can_compare(data_type: &DataType) -> bool {
             | DataType::LargeBinary
     )
 }
+
+// The list of operations currently supported.
+fn can_partial_eq(data_type: &DataType) -> bool {
+    can_partial_eq_and_ord(data_type)
+        || matches!(
+            data_type.to_logical_type(),
+            DataType::Interval(IntervalUnit::DayTime)
+                | DataType::Interval(IntervalUnit::MonthDayNano)
+        )
+}
