@@ -34,7 +34,7 @@ use crate::{
 /// See <https://arrow.apache.org/docs/format/CDataInterface.html#structure-definitions>
 /// This was created by bindgen
 #[repr(C)]
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct Ffi_ArrowArray {
     pub(crate) length: i64,
     pub(crate) null_count: i64,
@@ -53,6 +53,13 @@ pub struct Ffi_ArrowArray {
     private_data: *mut ::std::os::raw::c_void,
 }
 
+// Sound because the arrow specification does not allow multiple implementations
+// to change this struct
+// This is intrinsically impossible to prove because the implementations agree
+// on this as part of the Arrow specification
+unsafe impl Send for Ffi_ArrowArray {}
+unsafe impl Sync for Ffi_ArrowArray {}
+
 impl Drop for Ffi_ArrowArray {
     fn drop(&mut self) {
         match self.release {
@@ -331,7 +338,7 @@ fn create_dictionary(
     field: &Field,
     parent: Arc<ArrowArray>,
 ) -> Result<Option<ArrowArrayChild<'static>>> {
-    if let DataType::Dictionary(_, values) = field.data_type() {
+    if let DataType::Dictionary(_, values, _) = field.data_type() {
         let field = Field::new("", values.as_ref().clone(), true);
         assert!(!array.dictionary.is_null());
         let array = unsafe { &*array.dictionary };
