@@ -1,15 +1,16 @@
 use std::convert::TryInto;
 use std::sync::Arc;
 
-use avro_rs::Schema as AvroSchema;
+use avro_schema::{Enum, Schema as AvroSchema};
 
 use crate::array::*;
+use crate::chunk::Chunk;
 use crate::datatypes::*;
 use crate::error::ArrowError;
 use crate::error::Result;
-use crate::record_batch::RecordBatch;
 use crate::types::months_days_ns;
 
+use super::super::Block;
 use super::nested::*;
 use super::util;
 
@@ -33,7 +34,7 @@ fn make_mutable(
             Box::new(MutableUtf8Array::<i32>::with_capacity(capacity)) as Box<dyn MutableArray>
         }
         PhysicalType::Dictionary(_) => {
-            if let Some(AvroSchema::Enum { symbols, .. }) = avro_schema {
+            if let Some(AvroSchema::Enum(Enum { symbols, .. })) = avro_schema {
                 let values = Utf8Array::<i32>::from_slice(symbols);
                 Box::new(FixedItemsUtf8Dictionary::with_capacity(values, capacity))
                     as Box<dyn MutableArray>
@@ -64,7 +65,7 @@ fn make_mutable(
 
 fn is_union_null_first(avro_field: &AvroSchema) -> bool {
     if let AvroSchema::Union(schemas) = avro_field {
-        schemas.variants()[0] == AvroSchema::Null
+        schemas[0] == AvroSchema::Null
     } else {
         unreachable!()
     }
@@ -95,13 +96,18 @@ fn deserialize_value<'a>(
     let data_type = array.data_type();
     match data_type {
         DataType::List(inner) => {
-            let avro_inner = if let AvroSchema::Array(inner) = avro_field {
-                inner.as_ref()
-            } else {
-                unreachable!()
+            let avro_inner = match avro_field {
+                AvroSchema::Array(inner) => inner.as_ref(),
+                AvroSchema::Union(u) => match &u.as_slice() {
+                    &[AvroSchema::Array(inner), _] | &[_, AvroSchema::Array(inner)] => {
+                        inner.as_ref()
+                    }
+                    _ => unreachable!(),
+                },
+                _ => unreachable!(),
             };
 
-            let is_nullable = inner.is_nullable();
+            let is_nullable = inner.is_nullable;
             let array = array
                 .as_mut_any()
                 .downcast_mut::<DynMutableListArray<i32>>()
@@ -241,15 +247,17 @@ fn deserialize_value<'a>(
     Ok(block)
 }
 
+/// Deserializes a [`Block`] into [`Chunk`].
 pub fn deserialize(
-    mut block: &[u8],
-    rows: usize,
-    schema: Arc<Schema>,
+    block: &Block,
+    fields: &[Field],
     avro_schemas: &[AvroSchema],
-) -> Result<RecordBatch> {
+) -> Result<Chunk<Arc<dyn Array>>> {
+    let rows = block.number_of_rows;
+    let mut block = block.data.as_ref();
+
     // create mutables, one per field
-    let mut arrays: Vec<Box<dyn MutableArray>> = schema
-        .fields()
+    let mut arrays: Vec<Box<dyn MutableArray>> = fields
         .iter()
         .zip(avro_schemas.iter())
         .map(|(field, avro_schema)| {
@@ -262,13 +270,11 @@ pub fn deserialize(
     for _ in 0..rows {
         for ((array, field), avro_field) in arrays
             .iter_mut()
-            .zip(schema.fields().iter())
+            .zip(fields.iter())
             .zip(avro_schemas.iter())
         {
-            block = deserialize_item(array.as_mut(), field.is_nullable(), avro_field, block)?
+            block = deserialize_item(array.as_mut(), field.is_nullable, avro_field, block)?
         }
     }
-    let columns = arrays.iter_mut().map(|array| array.as_arc()).collect();
-
-    RecordBatch::try_new(schema, columns)
+    Chunk::try_new(arrays.iter_mut().map(|array| array.as_arc()).collect())
 }
