@@ -1,12 +1,16 @@
 #![deny(missing_docs)]
-//! Transfer data between the Arrow memory format and CSV (comma-separated values).
+//! Convert data between the Arrow and CSV (comma-separated values).
 
 use crate::error::ArrowError;
 
-pub use csv::Error as CSVError;
+#[cfg(any(feature = "io_csv_read_async", feature = "io_csv_read"))]
+mod read_utils;
+#[cfg(any(feature = "io_csv_read_async", feature = "io_csv_read"))]
+mod utils;
 
-impl From<CSVError> for ArrowError {
-    fn from(error: CSVError) -> Self {
+#[cfg(any(feature = "io_csv_read", feature = "io_csv_write"))]
+impl From<csv::Error> for ArrowError {
+    fn from(error: csv::Error) -> Self {
         ArrowError::External("".to_string(), Box::new(error))
     }
 }
@@ -23,3 +27,7 @@ pub mod read;
 #[cfg(feature = "io_csv_write")]
 #[cfg_attr(docsrs, doc(cfg(feature = "io_csv_write")))]
 pub mod write;
+
+#[cfg(feature = "io_csv_read_async")]
+#[cfg_attr(docsrs, doc(cfg(feature = "io_csv_read_async")))]
+pub mod read_async;
