@@ -1,33 +1,41 @@
-use std::collections::{HashMap, VecDeque};
+use std::collections::VecDeque;
 use std::io::{Read, Seek};
-use std::sync::Arc;
 
-use arrow_format::ipc;
-
-use crate::array::{Array, UnionArray};
+use crate::array::UnionArray;
 use crate::datatypes::DataType;
 use crate::datatypes::UnionMode::Dense;
-use crate::error::Result;
+use crate::error::{ArrowError, Result};
 
-use super::super::deserialize::{read, skip, Node};
+use super::super::super::IpcField;
+use super::super::deserialize::{read, skip};
 use super::super::read_basic::*;
+use super::super::Dictionaries;
+use super::super::{Compression, IpcBuffer, Node, Version};
 
 #[allow(clippy::too_many_arguments)]
 pub fn read_union<R: Read + Seek>(
     field_nodes: &mut VecDeque<Node>,
     data_type: DataType,
-    buffers: &mut VecDeque<&ipc::Schema::Buffer>,
+    ipc_field: &IpcField,
+    buffers: &mut VecDeque<IpcBuffer>,
     reader: &mut R,
-    dictionaries: &HashMap<usize, Arc<dyn Array>>,
+    dictionaries: &Dictionaries,
     block_offset: u64,
     is_little_endian: bool,
-    compression: Option<ipc::Message::BodyCompression>,
-    version: ipc::Schema::MetadataVersion,
+    compression: Option<Compression>,
+    version: Version,
 ) -> Result<UnionArray> {
-    let field_node = field_nodes.pop_front().unwrap();
+    let field_node = field_nodes.pop_front().ok_or_else(|| {
+        ArrowError::oos(format!(
+            "IPC: unable to fetch the field for {:?}. The file or stream is corrupted.",
+            data_type
+        ))
+    })?;
 
-    if version != ipc::Schema::MetadataVersion::V5 {
-        let _ = buffers.pop_front().unwrap();
+    if version != Version::V5 {
+        let _ = buffers
+            .pop_front()
+            .ok_or_else(|| ArrowError::oos("IPC: missing validity buffer."))?;
     };
 
     let types = read_buffer(
@@ -53,17 +61,19 @@ pub fn read_union<R: Read + Seek>(
             None
         }
     } else {
-        panic!()
+        unreachable!()
     };
 
     let fields = UnionArray::get_fields(&data_type);
 
     let fields = fields
         .iter()
-        .map(|field| {
+        .zip(ipc_field.fields.iter())
+        .map(|(field, ipc_field)| {
             read(
                 field_nodes,
                 field,
+                ipc_field,
                 buffers,
                 reader,
                 dictionaries,
@@ -81,20 +91,28 @@ pub fn read_union<R: Read + Seek>(
 pub fn skip_union(
     field_nodes: &mut VecDeque<Node>,
     data_type: &DataType,
-    buffers: &mut VecDeque<&ipc::Schema::Buffer>,
-) {
-    let _ = field_nodes.pop_front().unwrap();
+    buffers: &mut VecDeque<IpcBuffer>,
+) -> Result<()> {
+    let _ = field_nodes.pop_front().ok_or_else(|| {
+        ArrowError::oos(
+            "IPC: unable to fetch the field for struct. The file or stream is corrupted.",
+        )
+    })?;
 
-    let _ = buffers.pop_front().unwrap();
+    let _ = buffers
+        .pop_front()
+        .ok_or_else(|| ArrowError::oos("IPC: missing validity buffer."))?;
     if let DataType::Union(_, _, Dense) = data_type {
-        let _ = buffers.pop_front().unwrap();
+        let _ = buffers
+            .pop_front()
+            .ok_or_else(|| ArrowError::oos("IPC: missing offsets buffer."))?;
     } else {
-        panic!()
+        unreachable!()
     };
 
     let fields = UnionArray::get_fields(data_type);
 
     fields
         .iter()
-        .for_each(|field| skip(field_nodes, field.data_type(), buffers))
+        .try_for_each(|field| skip(field_nodes, field.data_type(), buffers))
 }
