@@ -1,33 +1,17 @@
-// Licensed to the Apache Software Foundation (ASF) under one
-// or more contributor license agreements.  See the NOTICE file
-// distributed with this work for additional information
-// regarding copyright ownership.  The ASF licenses this file
-// to you under the Apache License, Version 2.0 (the
-// "License"); you may not use this file except in compliance
-// with the License.  You may obtain a copy of the License at
-//
-//   http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing,
-// software distributed under the License is distributed on an
-// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-// KIND, either express or implied.  See the License for the
-// specific language governing permissions and limitations
-// under the License.
-
-use std::collections::{HashMap, VecDeque};
+use std::collections::VecDeque;
 use std::io::{Read, Seek};
 use std::sync::Arc;
 
-use arrow_format::ipc;
-use arrow_format::ipc::Schema::MetadataVersion;
+use arrow_format;
 
 use crate::array::*;
-use crate::datatypes::{DataType, Field, Schema};
+use crate::chunk::Chunk;
+use crate::datatypes::{DataType, Field};
 use crate::error::{ArrowError, Result};
-use crate::record_batch::RecordBatch;
+use crate::io::ipc::{IpcField, IpcSchema};
 
 use super::deserialize::{read, skip};
+use super::Dictionaries;
 
 type ArrayRef = Arc<dyn Array>;
 
@@ -94,93 +78,89 @@ impl<'a, A, I: Iterator<Item = A>> Iterator for ProjectionIter<'a, A, I> {
 /// Panics iff the projection is not in increasing order (e.g. `[1, 0]` nor `[0, 1, 1]` are valid)
 #[allow(clippy::too_many_arguments)]
 pub fn read_record_batch<R: Read + Seek>(
-    batch: ipc::Message::RecordBatch,
-    schema: Arc<Schema>,
-    projection: Option<(&[usize], Arc<Schema>)>,
-    is_little_endian: bool,
-    dictionaries: &HashMap<usize, Arc<dyn Array>>,
-    version: MetadataVersion,
+    batch: arrow_format::ipc::RecordBatchRef,
+    fields: &[Field],
+    ipc_schema: &IpcSchema,
+    projection: Option<&[usize]>,
+    dictionaries: &Dictionaries,
+    version: arrow_format::ipc::MetadataVersion,
     reader: &mut R,
     block_offset: u64,
-) -> Result<RecordBatch> {
-    let buffers = batch.buffers().ok_or_else(|| {
-        ArrowError::OutOfSpec("Unable to get buffers from IPC RecordBatch".to_string())
-    })?;
-    let mut buffers: VecDeque<&ipc::Schema::Buffer> = buffers.iter().collect();
-    let field_nodes = batch.nodes().ok_or_else(|| {
-        ArrowError::OutOfSpec("Unable to get field nodes from IPC RecordBatch".to_string())
-    })?;
+) -> Result<Chunk<Arc<dyn Array>>> {
+    assert_eq!(fields.len(), ipc_schema.fields.len());
+    let buffers = batch
+        .buffers()?
+        .ok_or_else(|| ArrowError::oos("IPC RecordBatch must contain buffers"))?;
+    let mut buffers: VecDeque<arrow_format::ipc::BufferRef> = buffers.iter().collect();
 
+    let field_nodes = batch
+        .nodes()?
+        .ok_or_else(|| ArrowError::oos("IPC RecordBatch must contain field nodes"))?;
     let mut field_nodes = field_nodes.iter().collect::<VecDeque<_>>();
 
-    let (schema, columns) = if let Some(projection) = projection {
-        let projected_schema = projection.1.clone();
+    let columns = if let Some(projection) = projection {
+        let projection =
+            ProjectionIter::new(projection, fields.iter().zip(ipc_schema.fields.iter()));
 
-        let projection = ProjectionIter::new(projection.0, schema.fields().iter());
-
-        let arrays = projection
+        projection
             .map(|maybe_field| match maybe_field {
-                ProjectionResult::Selected(field) => Some(read(
+                ProjectionResult::Selected((field, ipc_field)) => Ok(Some(read(
                     &mut field_nodes,
                     field,
+                    ipc_field,
                     &mut buffers,
                     reader,
                     dictionaries,
                     block_offset,
-                    is_little_endian,
-                    batch.compression(),
+                    ipc_schema.is_little_endian,
+                    batch.compression()?,
                     version,
-                )),
-                ProjectionResult::NotSelected(field) => {
-                    skip(&mut field_nodes, field.data_type(), &mut buffers);
-                    None
+                )?)),
+                ProjectionResult::NotSelected((field, _)) => {
+                    skip(&mut field_nodes, &field.data_type, &mut buffers)?;
+                    Ok(None)
                 }
             })
+            .map(|x| x.transpose())
             .flatten()
-            .collect::<Result<Vec<_>>>()?;
-        (projected_schema, arrays)
+            .collect::<Result<Vec<_>>>()?
     } else {
-        let arrays = schema
-            .fields()
+        fields
             .iter()
-            .map(|field| {
+            .zip(ipc_schema.fields.iter())
+            .map(|(field, ipc_field)| {
                 read(
                     &mut field_nodes,
                     field,
+                    ipc_field,
                     &mut buffers,
                     reader,
                     dictionaries,
                     block_offset,
-                    is_little_endian,
-                    batch.compression(),
+                    ipc_schema.is_little_endian,
+                    batch.compression()?,
                     version,
                 )
             })
-            .collect::<Result<Vec<_>>>()?;
-        (schema.clone(), arrays)
+            .collect::<Result<Vec<_>>>()?
     };
-    RecordBatch::try_new(schema, columns)
+    Chunk::try_new(columns)
 }
 
-fn find_first_dict_field_d(id: usize, data_type: &DataType) -> Option<&Field> {
+fn find_first_dict_field_d<'a>(
+    id: i64,
+    data_type: &'a DataType,
+    ipc_field: &'a IpcField,
+) -> Option<(&'a Field, &'a IpcField)> {
     use DataType::*;
     match data_type {
-        Dictionary(_, inner) => find_first_dict_field_d(id, inner.as_ref()),
-        Map(field, _) => find_first_dict_field(id, field.as_ref()),
-        List(field) => find_first_dict_field(id, field.as_ref()),
-        LargeList(field) => find_first_dict_field(id, field.as_ref()),
-        FixedSizeList(field, _) => find_first_dict_field(id, field.as_ref()),
-        Union(fields, _, _) => {
-            for field in fields {
-                if let Some(f) = find_first_dict_field(id, field) {
-                    return Some(f);
-                }
-            }
-            None
+        Dictionary(_, inner, _) => find_first_dict_field_d(id, inner.as_ref(), ipc_field),
+        List(field) | LargeList(field) | FixedSizeList(field, ..) | Map(field, ..) => {
+            find_first_dict_field(id, field.as_ref(), &ipc_field.fields[0])
         }
-        Struct(fields) => {
-            for field in fields {
-                if let Some(f) = find_first_dict_field(id, field) {
+        Union(fields, ..) | Struct(fields) => {
+            for (field, ipc_field) in fields.iter().zip(ipc_field.fields.iter()) {
+                if let Some(f) = find_first_dict_field(id, field, ipc_field) {
                     return Some(f);
                 }
             }
@@ -190,18 +170,27 @@ fn find_first_dict_field_d(id: usize, data_type: &DataType) -> Option<&Field> {
     }
 }
 
-fn find_first_dict_field(id: usize, field: &Field) -> Option<&Field> {
-    if let DataType::Dictionary(_, _) = &field.data_type {
-        if field.dict_id as usize == id {
-            return Some(field);
+fn find_first_dict_field<'a>(
+    id: i64,
+    field: &'a Field,
+    ipc_field: &'a IpcField,
+) -> Option<(&'a Field, &'a IpcField)> {
+    if let Some(field_id) = ipc_field.dictionary_id {
+        if id == field_id {
+            return Some((field, ipc_field));
         }
     }
-    find_first_dict_field_d(id, &field.data_type)
+    find_first_dict_field_d(id, &field.data_type, ipc_field)
 }
 
-fn first_dict_field(id: usize, fields: &[Field]) -> Result<&Field> {
-    for field in fields {
-        if let Some(field) = find_first_dict_field(id, field) {
+fn first_dict_field<'a>(
+    id: i64,
+    fields: &'a [Field],
+    ipc_fields: &'a [IpcField],
+) -> Result<(&'a Field, &'a IpcField)> {
+    assert_eq!(fields.len(), ipc_fields.len());
+    for (field, ipc_field) in fields.iter().zip(ipc_fields.iter()) {
+        if let Some(field) = find_first_dict_field(id, field, ipc_field) {
             return Ok(field);
         }
     }
@@ -214,44 +203,47 @@ fn first_dict_field(id: usize, fields: &[Field]) -> Result<&Field> {
 /// Read the dictionary from the buffer and provided metadata,
 /// updating the `dictionaries` with the resulting dictionary
 pub fn read_dictionary<R: Read + Seek>(
-    batch: ipc::Message::DictionaryBatch,
-    schema: &Schema,
-    is_little_endian: bool,
-    dictionaries: &mut HashMap<usize, Arc<dyn Array>>,
+    batch: arrow_format::ipc::DictionaryBatchRef,
+    fields: &[Field],
+    ipc_schema: &IpcSchema,
+    dictionaries: &mut Dictionaries,
     reader: &mut R,
     block_offset: u64,
 ) -> Result<()> {
-    if batch.isDelta() {
+    if batch.is_delta()? {
         return Err(ArrowError::NotYetImplemented(
             "delta dictionary batches not supported".to_string(),
         ));
     }
 
-    let id = batch.id();
-    let first_field = first_dict_field(id as usize, &schema.fields)?;
+    let id = batch.id()?;
+    let (first_field, first_ipc_field) = first_dict_field(id, fields, &ipc_schema.fields)?;
 
     // As the dictionary batch does not contain the type of the
     // values array, we need to retrieve this from the schema.
     // Get an array representing this dictionary's values.
-    let dictionary_values: ArrayRef = match first_field.data_type() {
-        DataType::Dictionary(_, ref value_type) => {
+    let dictionary_values: ArrayRef = match &first_field.data_type {
+        DataType::Dictionary(_, ref value_type, _) => {
             // Make a fake schema for the dictionary batch.
-            let schema = Arc::new(Schema {
-                fields: vec![Field::new("", value_type.as_ref().clone(), false)],
-                metadata: HashMap::new(),
-            });
-            // Read a single column
-            let record_batch = read_record_batch(
-                batch.data().unwrap(),
-                schema,
+            let fields = vec![Field::new("", value_type.as_ref().clone(), false)];
+            let ipc_schema = IpcSchema {
+                fields: vec![first_ipc_field.clone()],
+                is_little_endian: ipc_schema.is_little_endian,
+            };
+            let columns = read_record_batch(
+                batch
+                    .data()?
+                    .ok_or_else(|| ArrowError::oos("The dictionary batch must have data."))?,
+                &fields,
+                &ipc_schema,
                 None,
-                is_little_endian,
                 dictionaries,
-                MetadataVersion::V5,
+                arrow_format::ipc::MetadataVersion::V5,
                 reader,
                 block_offset,
             )?;
-            Some(record_batch.column(0).clone())
+            let mut arrays = columns.into_arrays();
+            Some(arrays.pop().unwrap())
         }
         _ => None,
     }
@@ -259,7 +251,7 @@ pub fn read_dictionary<R: Read + Seek>(
         ArrowError::InvalidArgumentError("dictionary id not found in schema".to_string())
     })?;
 
-    dictionaries.insert(id as usize, dictionary_values);
+    dictionaries.insert(id, dictionary_values);
 
     Ok(())
 }
