@@ -1,60 +1,34 @@
-// Licensed to the Apache Software Foundation (ASF) under one
-// or more contributor license agreements.  See the NOTICE file
-// distributed with this work for additional information
-// regarding copyright ownership.  The ASF licenses this file
-// to you under the Apache License, Version 2.0 (the
-// "License"); you may not use this file except in compliance
-// with the License.  You may obtain a copy of the License at
-//
-//   http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing,
-// software distributed under the License is distributed on an
-// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-// KIND, either express or implied.  See the License for the
-// specific language governing permissions and limitations
-// under the License.
-
-use std::collections::HashMap;
+use std::convert::TryInto;
 use std::io::{Read, Seek, SeekFrom};
 use std::sync::Arc;
 
-use arrow_format::ipc;
-use arrow_format::ipc::flatbuffers::VerifierOptions;
-
-use crate::array::*;
-use crate::datatypes::Schema;
+use crate::array::Array;
+use crate::chunk::Chunk;
+use crate::datatypes::{Field, Schema};
 use crate::error::{ArrowError, Result};
-use crate::record_batch::RecordBatch;
+use crate::io::ipc::IpcSchema;
 
-use super::super::convert;
 use super::super::{ARROW_MAGIC, CONTINUATION_MARKER};
 use super::common::*;
+use super::schema::fb_to_schema;
+use super::Dictionaries;
+use arrow_format::ipc::planus::{ReadAsRoot, Vector};
 
 #[derive(Debug, Clone)]
 pub struct FileMetadata {
-    /// The schema that is read from the file header
-    schema: Arc<Schema>,
+    /// The schema that is read from the file footer
+    pub schema: Schema,
+
+    /// The files' [`IpcSchema`]
+    pub ipc_schema: IpcSchema,
 
     /// The blocks in the file
     ///
     /// A block indicates the regions in the file to read to get data
-    blocks: Vec<ipc::File::Block>,
+    blocks: Vec<arrow_format::ipc::Block>,
 
     /// Dictionaries associated to each dict_id
-    dictionaries: HashMap<usize, Arc<dyn Array>>,
-
-    /// FileMetadata version
-    version: ipc::Schema::MetadataVersion,
-
-    is_little_endian: bool,
-}
-
-impl FileMetadata {
-    /// Returns the schema.
-    pub fn schema(&self) -> &Arc<Schema> {
-        &self.schema
-    }
+    dictionaries: Dictionaries,
 }
 
 /// Arrow File reader
@@ -62,7 +36,7 @@ pub struct FileReader<R: Read + Seek> {
     reader: R,
     metadata: FileMetadata,
     current_block: usize,
-    projection: Option<(Vec<usize>, Arc<Schema>)>,
+    projection: Option<(Vec<usize>, Schema)>,
     buffer: Vec<u8>,
 }
 
@@ -87,6 +61,51 @@ fn read_dictionary_message<R: Read + Seek>(
     Ok(())
 }
 
+fn read_dictionaries<R: Read + Seek>(
+    reader: &mut R,
+    fields: &[Field],
+    ipc_schema: &IpcSchema,
+    blocks: Vector<arrow_format::ipc::BlockRef>,
+) -> Result<Dictionaries> {
+    let mut dictionaries = Default::default();
+    let mut data = vec![];
+
+    for block in blocks {
+        let offset = block.offset() as u64;
+        let length = block.meta_data_length() as u64;
+        read_dictionary_message(reader, offset, &mut data)?;
+
+        let message = arrow_format::ipc::MessageRef::read_as_root(&data).map_err(|err| {
+            ArrowError::OutOfSpec(format!("Unable to get root as message: {:?}", err))
+        })?;
+
+        let header = message
+            .header()?
+            .ok_or_else(|| ArrowError::oos("Message must have an header"))?;
+
+        match header {
+            arrow_format::ipc::MessageHeaderRef::DictionaryBatch(batch) => {
+                let block_offset = offset + length;
+                read_dictionary(
+                    batch,
+                    fields,
+                    ipc_schema,
+                    &mut dictionaries,
+                    reader,
+                    block_offset,
+                )?;
+            }
+            t => {
+                return Err(ArrowError::OutOfSpec(format!(
+                    "Expecting DictionaryBatch in dictionary blocks, found {:?}.",
+                    t
+                )));
+            }
+        };
+    }
+    Ok(dictionaries)
+}
+
 /// Read the IPC file's metadata
 pub fn read_file_metadata<R: Read + Seek>(reader: &mut R) -> Result<FileMetadata> {
     // check if header and footer contain correct magic bytes
@@ -115,87 +134,48 @@ pub fn read_file_metadata<R: Read + Seek>(reader: &mut R) -> Result<FileMetadata
     reader.seek(SeekFrom::End(-10 - footer_len as i64))?;
     reader.read_exact(&mut footer_data)?;
 
-    // set flatbuffer verification options to the same settings as the C++ arrow implementation.
-    // Heuristic: tables in a Arrow flatbuffers buffer must take at least 1 bit
-    // each in average (ARROW-11559).
-    // Especially, the only recursive table (the `Field` table in Schema.fbs)
-    // must have a non-empty `type` member.
-    let verifier_options = VerifierOptions {
-        max_depth: 128,
-        max_tables: footer_len as usize * 8,
-        ..Default::default()
-    };
-    let footer = ipc::File::root_as_footer_with_opts(&verifier_options, &footer_data[..])
+    let footer = arrow_format::ipc::FooterRef::read_as_root(&footer_data)
         .map_err(|err| ArrowError::OutOfSpec(format!("Unable to get root as footer: {:?}", err)))?;
 
-    let blocks = footer.recordBatches().ok_or_else(|| {
+    let blocks = footer.record_batches()?.ok_or_else(|| {
         ArrowError::OutOfSpec("Unable to get record batches from footer".to_string())
     })?;
 
     let ipc_schema = footer
-        .schema()
+        .schema()?
         .ok_or_else(|| ArrowError::OutOfSpec("Unable to get the schema from footer".to_string()))?;
-    let (schema, is_little_endian) = convert::fb_to_schema(ipc_schema);
-    let schema = Arc::new(schema);
+    let (schema, ipc_schema) = fb_to_schema(ipc_schema)?;
 
-    let mut dictionaries = Default::default();
+    let dictionary_blocks = footer.dictionaries()?;
 
-    let dictionary_blocks = footer.dictionaries().ok_or_else(|| {
-        ArrowError::OutOfSpec("Unable to get dictionaries from footer".to_string())
-    })?;
+    let dictionaries = if let Some(blocks) = dictionary_blocks {
+        read_dictionaries(reader, &schema.fields, &ipc_schema, blocks)?
+    } else {
+        Default::default()
+    };
 
-    let mut data = vec![];
-    for block in dictionary_blocks {
-        let offset = block.offset() as u64;
-        let length = block.metaDataLength() as u64;
-        read_dictionary_message(reader, offset, &mut data)?;
-
-        let message = ipc::Message::root_as_message(&data).map_err(|err| {
-            ArrowError::OutOfSpec(format!("Unable to get root as message: {:?}", err))
-        })?;
-
-        match message.header_type() {
-            ipc::Message::MessageHeader::DictionaryBatch => {
-                let block_offset = offset + length;
-                let batch = message.header_as_dictionary_batch().unwrap();
-                read_dictionary(
-                    batch,
-                    &schema,
-                    is_little_endian,
-                    &mut dictionaries,
-                    reader,
-                    block_offset,
-                )?;
-            }
-            t => {
-                return Err(ArrowError::OutOfSpec(format!(
-                    "Expecting DictionaryBatch in dictionary blocks, found {:?}.",
-                    t
-                )));
-            }
-        };
-    }
     Ok(FileMetadata {
         schema,
-        is_little_endian,
-        blocks: blocks.to_vec(),
+        ipc_schema,
+        blocks: blocks
+            .iter()
+            .map(|block| Ok(block.try_into()?))
+            .collect::<Result<Vec<_>>>()?,
         dictionaries,
-        version: footer.version(),
     })
 }
 
 fn get_serialized_batch<'a>(
-    message: &'a ipc::Message::Message,
-) -> Result<ipc::Message::RecordBatch<'a>> {
-    match message.header_type() {
-        ipc::Message::MessageHeader::Schema => Err(ArrowError::OutOfSpec(
+    message: &'a arrow_format::ipc::MessageRef,
+) -> Result<arrow_format::ipc::RecordBatchRef<'a>> {
+    let header = message.header()?.ok_or_else(|| {
+        ArrowError::oos("IPC: unable to fetch the message header. The file or stream is corrupted.")
+    })?;
+    match header {
+        arrow_format::ipc::MessageHeaderRef::Schema(_) => Err(ArrowError::OutOfSpec(
             "Not expecting a schema when messages are read".to_string(),
         )),
-        ipc::Message::MessageHeader::RecordBatch => {
-            message.header_as_record_batch().ok_or_else(|| {
-                ArrowError::OutOfSpec("Unable to read IPC message as record batch".to_string())
-            })
-        }
+        arrow_format::ipc::MessageHeaderRef::RecordBatch(batch) => Ok(batch),
         t => Err(ArrowError::OutOfSpec(format!(
             "Reading types other than record batches not yet supported, unable to read {:?}",
             t
@@ -207,14 +187,14 @@ fn get_serialized_batch<'a>(
 pub fn read_batch<R: Read + Seek>(
     reader: &mut R,
     metadata: &FileMetadata,
-    projection: Option<(&[usize], Arc<Schema>)>,
+    projection: Option<&[usize]>,
     block: usize,
     block_data: &mut Vec<u8>,
-) -> Result<RecordBatch> {
+) -> Result<Chunk<Arc<dyn Array>>> {
     let block = metadata.blocks[block];
 
     // read length
-    reader.seek(SeekFrom::Start(block.offset() as u64))?;
+    reader.seek(SeekFrom::Start(block.offset as u64))?;
     let mut meta_buf = [0; 4];
     reader.read_exact(&mut meta_buf)?;
     if meta_buf == CONTINUATION_MARKER {
@@ -227,28 +207,20 @@ pub fn read_batch<R: Read + Seek>(
     block_data.resize(meta_len, 0);
     reader.read_exact(block_data)?;
 
-    let message = ipc::Message::root_as_message(&block_data[..])
-        .map_err(|err| ArrowError::OutOfSpec(format!("Unable to get root as footer: {:?}", err)))?;
-
-    // some old test data's footer metadata is not set, so we account for that
-    if metadata.version != ipc::Schema::MetadataVersion::V1 && message.version() != metadata.version
-    {
-        return Err(ArrowError::OutOfSpec(
-            "Could not read IPC message as metadata versions mismatch".to_string(),
-        ));
-    }
+    let message = arrow_format::ipc::MessageRef::read_as_root(&block_data[..])
+        .map_err(|err| ArrowError::oos(format!("Unable parse message: {:?}", err)))?;
 
     let batch = get_serialized_batch(&message)?;
 
     read_record_batch(
         batch,
-        metadata.schema.clone(),
+        &metadata.schema.fields,
+        &metadata.ipc_schema,
         projection,
-        metadata.is_little_endian,
         &metadata.dictionaries,
-        metadata.version,
+        message.version()?,
         reader,
-        block.offset() as u64 + block.metaDataLength() as u64,
+        block.offset as u64 + block.meta_data_length as u64,
     )
 }
 
@@ -266,12 +238,14 @@ impl<R: Read + Seek> FileReader<R> {
             });
         }
         let projection = projection.map(|projection| {
-            let fields = metadata.schema().fields();
-            let fields = projection.iter().map(|x| fields[*x].clone()).collect();
-            let schema = Arc::new(Schema {
+            let fields = projection
+                .iter()
+                .map(|x| metadata.schema.fields[*x].clone())
+                .collect();
+            let schema = Schema {
                 fields,
-                metadata: metadata.schema().metadata().clone(),
-            });
+                metadata: metadata.schema.metadata.clone(),
+            };
             (projection, schema)
         });
         Self {
@@ -284,13 +258,18 @@ impl<R: Read + Seek> FileReader<R> {
     }
 
     /// Return the schema of the file
-    pub fn schema(&self) -> &Arc<Schema> {
+    pub fn schema(&self) -> &Schema {
         self.projection
             .as_ref()
             .map(|x| &x.1)
             .unwrap_or(&self.metadata.schema)
     }
 
+    /// Returns the [`FileMetadata`]
+    pub fn metadata(&self) -> &FileMetadata {
+        &self.metadata
+    }
+
     /// Consumes this FileReader, returning the underlying reader
     pub fn into_inner(self) -> R {
         self.reader
@@ -298,7 +277,7 @@ impl<R: Read + Seek> FileReader<R> {
 }
 
 impl<R: Read + Seek> Iterator for FileReader<R> {
-    type Item = Result<RecordBatch>;
+    type Item = Result<Chunk<Arc<dyn Array>>>;
 
     fn next(&mut self) -> Option<Self::Item> {
         // get current block
@@ -308,9 +287,7 @@ impl<R: Read + Seek> Iterator for FileReader<R> {
             Some(read_batch(
                 &mut self.reader,
                 &self.metadata,
-                self.projection
-                    .as_ref()
-                    .map(|x| (x.0.as_ref(), x.1.clone())),
+                self.projection.as_ref().map(|x| x.0.as_ref()),
                 block,
                 &mut self.buffer,
             ))
