@@ -1,45 +1,28 @@
-// Licensed to the Apache Software Foundation (ASF) under one
-// or more contributor license agreements.  See the NOTICE file
-// distributed with this work for additional information
-// regarding copyright ownership.  The ASF licenses this file
-// to you under the Apache License, Version 2.0 (the
-// "License"); you may not use this file except in compliance
-// with the License.  You may obtain a copy of the License at
-//
-//   http://www.apache.org/licenses/LICENSE-2.0
-//
-// Unless required by applicable law or agreed to in writing,
-// software distributed under the License is distributed on an
-// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
-// KIND, either express or implied.  See the License for the
-// specific language governing permissions and limitations
-// under the License.
-
-use std::collections::HashMap;
 use std::io::Read;
 use std::sync::Arc;
 
-use arrow_format::ipc;
-use arrow_format::ipc::Schema::MetadataVersion;
+use arrow_format;
+use arrow_format::ipc::planus::ReadAsRoot;
 
-use crate::array::*;
+use crate::array::Array;
+use crate::chunk::Chunk;
 use crate::datatypes::Schema;
 use crate::error::{ArrowError, Result};
-use crate::record_batch::RecordBatch;
+use crate::io::ipc::IpcSchema;
 
-use super::super::convert;
 use super::super::CONTINUATION_MARKER;
 use super::common::*;
+use super::schema::fb_to_schema;
+use super::Dictionaries;
 
-#[derive(Debug)]
+#[derive(Debug, Clone)]
 pub struct StreamMetadata {
     /// The schema that is read from the stream's first message
-    schema: Arc<Schema>,
+    pub schema: Schema,
 
-    version: MetadataVersion,
+    pub version: arrow_format::ipc::MetadataVersion,
 
-    /// Whether the incoming stream is little-endian
-    is_little_endian: bool,
+    pub ipc_schema: IpcSchema,
 }
 
 /// Reads the metadata of the stream
@@ -59,21 +42,28 @@ pub fn read_stream_metadata<R: Read>(reader: &mut R) -> Result<StreamMetadata> {
     let mut meta_buffer = vec![0; meta_len as usize];
     reader.read_exact(&mut meta_buffer)?;
 
-    let message = ipc::Message::root_as_message(meta_buffer.as_slice()).map_err(|err| {
-        ArrowError::OutOfSpec(format!("Unable to get root as message: {:?}", err))
-    })?;
-    let version = message.version();
+    let message =
+        arrow_format::ipc::MessageRef::read_as_root(meta_buffer.as_slice()).map_err(|err| {
+            ArrowError::OutOfSpec(format!("Unable to get root as message: {:?}", err))
+        })?;
+    let version = message.version()?;
     // message header is a Schema, so read it
-    let ipc_schema: ipc::Schema::Schema = message
-        .header_as_schema()
-        .ok_or_else(|| ArrowError::OutOfSpec("Unable to read IPC message as schema".to_string()))?;
-    let (schema, is_little_endian) = convert::fb_to_schema(ipc_schema);
-    let schema = Arc::new(schema);
+    let header = message
+        .header()?
+        .ok_or_else(|| ArrowError::oos("Unable to read the first IPC message"))?;
+    let schema = if let arrow_format::ipc::MessageHeaderRef::Schema(schema) = header {
+        schema
+    } else {
+        return Err(ArrowError::oos(
+            "The first IPC message of the stream must be a schema",
+        ));
+    };
+    let (schema, ipc_schema) = fb_to_schema(schema)?;
 
     Ok(StreamMetadata {
         schema,
         version,
-        is_little_endian,
+        ipc_schema,
     })
 }
 
@@ -82,15 +72,15 @@ pub fn read_stream_metadata<R: Read>(reader: &mut R) -> Result<StreamMetadata> {
 /// A stream is an iterator, and an iterator returns `Option<Item>`. The `Item`
 /// type in the [`StreamReader`] case is `StreamState`, which means that an Arrow
 /// stream may yield one of three values: (1) `None`, which signals that the stream
-/// is done; (2) `Some(StreamState::Some(RecordBatch))`, which signals that there was
+/// is done; (2) [`StreamState::Some`], which signals that there was
 /// data waiting in the stream and we read it; and finally (3)
-/// `Some(StreamState::Waiting)`, which means that the stream is still "live", it
+/// [`Some(StreamState::Waiting)`], which means that the stream is still "live", it
 /// just doesn't hold any data right now.
 pub enum StreamState {
     /// A live stream without data
     Waiting,
     /// Next item in the stream
-    Some(RecordBatch),
+    Some(Chunk<Arc<dyn Array>>),
 }
 
 impl StreamState {
@@ -99,7 +89,7 @@ impl StreamState {
     /// # Panics
     ///
     /// If the `StreamState` was `Waiting`.
-    pub fn unwrap(self) -> RecordBatch {
+    pub fn unwrap(self) -> Chunk<Arc<dyn Array>> {
         if let StreamState::Some(batch) = self {
             batch
         } else {
@@ -113,7 +103,7 @@ impl StreamState {
 fn read_next<R: Read>(
     reader: &mut R,
     metadata: &StreamMetadata,
-    dictionaries: &mut HashMap<usize, Arc<dyn Array>>,
+    dictionaries: &mut Dictionaries,
     message_buffer: &mut Vec<u8>,
     data_buffer: &mut Vec<u8>,
 ) -> Result<Option<StreamState>> {
@@ -152,30 +142,28 @@ fn read_next<R: Read>(
     message_buffer.resize(meta_length, 0);
     reader.read_exact(message_buffer)?;
 
-    let message = ipc::Message::root_as_message(message_buffer).map_err(|err| {
+    let message = arrow_format::ipc::MessageRef::read_as_root(message_buffer).map_err(|err| {
         ArrowError::OutOfSpec(format!("Unable to get root as message: {:?}", err))
     })?;
+    let header = message.header()?.ok_or_else(|| {
+        ArrowError::oos("IPC: unable to fetch the message header. The file or stream is corrupted.")
+    })?;
 
-    match message.header_type() {
-        ipc::Message::MessageHeader::Schema => Err(ArrowError::OutOfSpec(
-            "Not expecting a schema when messages are read".to_string(),
-        )),
-        ipc::Message::MessageHeader::RecordBatch => {
-            let batch = message.header_as_record_batch().ok_or_else(|| {
-                ArrowError::OutOfSpec("Unable to read IPC message as record batch".to_string())
-            })?;
+    match header {
+        arrow_format::ipc::MessageHeaderRef::Schema(_) => Err(ArrowError::oos("A stream ")),
+        arrow_format::ipc::MessageHeaderRef::RecordBatch(batch) => {
             // read the block that makes up the record batch into a buffer
             data_buffer.clear();
-            data_buffer.resize(message.bodyLength() as usize, 0);
+            data_buffer.resize(message.body_length()? as usize, 0);
             reader.read_exact(data_buffer)?;
 
             let mut reader = std::io::Cursor::new(data_buffer);
 
             read_record_batch(
                 batch,
-                metadata.schema.clone(),
+                &metadata.schema.fields,
+                &metadata.ipc_schema,
                 None,
-                metadata.is_little_endian,
                 dictionaries,
                 metadata.version,
                 &mut reader,
@@ -183,29 +171,25 @@ fn read_next<R: Read>(
             )
             .map(|x| Some(StreamState::Some(x)))
         }
-        ipc::Message::MessageHeader::DictionaryBatch => {
-            let batch = message.header_as_dictionary_batch().ok_or_else(|| {
-                ArrowError::OutOfSpec("Unable to read IPC message as dictionary batch".to_string())
-            })?;
+        arrow_format::ipc::MessageHeaderRef::DictionaryBatch(batch) => {
             // read the block that makes up the dictionary batch into a buffer
-            let mut buf = vec![0; message.bodyLength() as usize];
+            let mut buf = vec![0; message.body_length()? as usize];
             reader.read_exact(&mut buf)?;
 
             let mut dict_reader = std::io::Cursor::new(buf);
 
             read_dictionary(
                 batch,
-                &metadata.schema,
-                metadata.is_little_endian,
+                &metadata.schema.fields,
+                &metadata.ipc_schema,
                 dictionaries,
                 &mut dict_reader,
                 0,
             )?;
 
-            // read the next message until we encounter a RecordBatch
+            // read the next message until we encounter a RecordBatch message
             read_next(reader, metadata, dictionaries, message_buffer, data_buffer)
         }
-        ipc::Message::MessageHeader::NONE => Ok(Some(StreamState::Waiting)),
         t => Err(ArrowError::OutOfSpec(format!(
             "Reading types other than record batches not yet supported, unable to read {:?} ",
             t
@@ -222,7 +206,7 @@ fn read_next<R: Read>(
 pub struct StreamReader<R: Read> {
     reader: R,
     metadata: StreamMetadata,
-    dictionaries: HashMap<usize, Arc<dyn Array>>,
+    dictionaries: Dictionaries,
     finished: bool,
     data_buffer: Vec<u8>,
     message_buffer: Vec<u8>,
@@ -246,8 +230,8 @@ impl<R: Read> StreamReader<R> {
     }
 
     /// Return the schema of the stream
-    pub fn schema(&self) -> &Arc<Schema> {
-        &self.metadata.schema
+    pub fn metadata(&self) -> &StreamMetadata {
+        &self.metadata
     }
 
     /// Check if the stream is finished
