@@ -1,15 +1,15 @@
-use std::{collections::HashMap, sync::Arc};
+use std::sync::Arc;
 
-use arrow_format::ipc;
-use arrow_format::ipc::flatbuffers::FlatBufferBuilder;
-use arrow_format::ipc::Message::CompressionType;
+use arrow_format::ipc::planus::Builder;
 
 use crate::array::*;
+use crate::chunk::Chunk;
 use crate::datatypes::*;
 use crate::error::{ArrowError, Result};
 use crate::io::ipc::endianess::is_native_little_endian;
-use crate::record_batch::RecordBatch;
+use crate::io::ipc::read::Dictionaries;
 
+use super::super::IpcField;
 use super::{write, write_dictionary};
 
 /// Compression codec
@@ -30,7 +30,7 @@ pub struct WriteOptions {
 }
 
 fn encode_dictionary(
-    field: &Field,
+    field: &IpcField,
     array: &Arc<dyn Array>,
     options: &WriteOptions,
     dictionary_tracker: &mut DictionaryTracker,
@@ -41,14 +41,11 @@ fn encode_dictionary(
         Utf8 | LargeUtf8 | Binary | LargeBinary | Primitive(_) | Boolean | Null
         | FixedSizeBinary => Ok(()),
         Dictionary(key_type) => match_integer_type!(key_type, |$T| {
-            let dict_id = field
-                .dict_id()
-                .expect("All Dictionary types have `dict_id`");
+            let dict_id = field.dictionary_id
+                .ok_or_else(|| ArrowError::InvalidArgumentError("Dictionaries must have an associated id".to_string()))?;
 
             let values = array.as_any().downcast_ref::<DictionaryArray<$T>>().unwrap().values();
-            // todo: this is won't work for Dict<Dict<...>>;
-            let field = Field::new("item", values.data_type().clone(), true);
-            encode_dictionary(&field,
+            encode_dictionary(field,
                 values,
                 options,
                 dictionary_tracker,
@@ -68,19 +65,16 @@ fn encode_dictionary(
             Ok(())
         }),
         Struct => {
-            let values = array
-                .as_any()
-                .downcast_ref::<StructArray>()
-                .unwrap()
-                .values();
-            let fields = if let DataType::Struct(fields) = array.data_type() {
-                fields
-            } else {
-                unreachable!()
-            };
+            let array = array.as_any().downcast_ref::<StructArray>().unwrap();
+            let fields = field.fields.as_slice();
+            if array.fields().len() != fields.len() {
+                return Err(ArrowError::InvalidArgumentError(
+                    "The number of fields in a struct must equal the number of children in IpcField".to_string(),
+                ));
+            }
             fields
                 .iter()
-                .zip(values.iter())
+                .zip(array.values().iter())
                 .try_for_each(|(field, values)| {
                     encode_dictionary(
                         field,
@@ -97,11 +91,7 @@ fn encode_dictionary(
                 .downcast_ref::<ListArray<i32>>()
                 .unwrap()
                 .values();
-            let field = if let DataType::List(field) = field.data_type() {
-                field.as_ref()
-            } else {
-                unreachable!()
-            };
+            let field = &field.fields[0]; // todo: error instead
             encode_dictionary(
                 field,
                 values,
@@ -116,11 +106,7 @@ fn encode_dictionary(
                 .downcast_ref::<ListArray<i64>>()
                 .unwrap()
                 .values();
-            let field = if let DataType::LargeList(field) = field.data_type() {
-                field.as_ref()
-            } else {
-                unreachable!()
-            };
+            let field = &field.fields[0]; // todo: error instead
             encode_dictionary(
                 field,
                 values,
@@ -135,11 +121,7 @@ fn encode_dictionary(
                 .downcast_ref::<FixedSizeListArray>()
                 .unwrap()
                 .values();
-            let field = if let DataType::FixedSizeList(field, _) = field.data_type() {
-                field.as_ref()
-            } else {
-                unreachable!()
-            };
+            let field = &field.fields[0]; // todo: error instead
             encode_dictionary(
                 field,
                 values,
@@ -154,11 +136,13 @@ fn encode_dictionary(
                 .downcast_ref::<UnionArray>()
                 .unwrap()
                 .fields();
-            let fields = if let DataType::Union(fields, _, _) = field.data_type() {
-                fields
-            } else {
-                unreachable!()
-            };
+            let fields = &field.fields[..]; // todo: error instead
+            if values.len() != fields.len() {
+                return Err(ArrowError::InvalidArgumentError(
+                    "The number of fields in a union must equal the number of children in IpcField"
+                        .to_string(),
+                ));
+            }
             fields
                 .iter()
                 .zip(values.iter())
@@ -174,11 +158,7 @@ fn encode_dictionary(
         }
         Map => {
             let values = array.as_any().downcast_ref::<MapArray>().unwrap().field();
-            let field = if let DataType::Map(field, _) = field.data_type() {
-                field.as_ref()
-            } else {
-                unreachable!()
-            };
+            let field = &field.fields[0]; // todo: error instead
             encode_dictionary(
                 field,
                 values,
@@ -190,39 +170,54 @@ fn encode_dictionary(
     }
 }
 
-pub fn encoded_batch(
-    batch: &RecordBatch,
+pub fn encode_chunk(
+    columns: &Chunk<Arc<dyn Array>>,
+    fields: &[IpcField],
     dictionary_tracker: &mut DictionaryTracker,
     options: &WriteOptions,
 ) -> Result<(Vec<EncodedData>, EncodedData)> {
-    let schema = batch.schema();
-    let mut encoded_dictionaries = Vec::with_capacity(schema.fields().len());
+    let mut encoded_dictionaries = vec![];
 
-    for (field, column) in schema.fields().iter().zip(batch.columns()) {
+    for (field, array) in fields.iter().zip(columns.as_ref()) {
         encode_dictionary(
             field,
-            column,
+            array,
             options,
             dictionary_tracker,
             &mut encoded_dictionaries,
         )?;
     }
 
-    let encoded_message = record_batch_to_bytes(batch, options);
+    let encoded_message = columns_to_bytes(columns, options);
 
     Ok((encoded_dictionaries, encoded_message))
 }
 
-/// Write a `RecordBatch` into two sets of bytes, one for the header (ipc::Schema::Message) and the
-/// other for the batch's data
-fn record_batch_to_bytes(batch: &RecordBatch, options: &WriteOptions) -> EncodedData {
-    let mut fbb = FlatBufferBuilder::new();
+fn serialize_compression(
+    compression: Option<Compression>,
+) -> Option<Box<arrow_format::ipc::BodyCompression>> {
+    if let Some(compression) = compression {
+        let codec = match compression {
+            Compression::LZ4 => arrow_format::ipc::CompressionType::Lz4Frame,
+            Compression::ZSTD => arrow_format::ipc::CompressionType::Zstd,
+        };
+        Some(Box::new(arrow_format::ipc::BodyCompression {
+            codec,
+            method: arrow_format::ipc::BodyCompressionMethod::Buffer,
+        }))
+    } else {
+        None
+    }
+}
 
-    let mut nodes: Vec<ipc::Message::FieldNode> = vec![];
-    let mut buffers: Vec<ipc::Schema::Buffer> = vec![];
+/// Write [`Chunk`] into two sets of bytes, one for the header (ipc::Schema::Message) and the
+/// other for the batch's data
+fn columns_to_bytes(columns: &Chunk<Arc<dyn Array>>, options: &WriteOptions) -> EncodedData {
+    let mut nodes: Vec<arrow_format::ipc::FieldNode> = vec![];
+    let mut buffers: Vec<arrow_format::ipc::Buffer> = vec![];
     let mut arrow_data: Vec<u8> = vec![];
     let mut offset = 0;
-    for array in batch.columns() {
+    for array in columns.arrays() {
         write(
             array.as_ref(),
             &mut buffers,
@@ -234,45 +229,27 @@ fn record_batch_to_bytes(batch: &RecordBatch, options: &WriteOptions) -> Encoded
         )
     }
 
-    // write data
-    let buffers = fbb.create_vector(&buffers);
-    let nodes = fbb.create_vector(&nodes);
+    let compression = serialize_compression(options.compression);
 
-    let compression = if let Some(compression) = options.compression {
-        let compression = match compression {
-            Compression::LZ4 => CompressionType::LZ4_FRAME,
-            Compression::ZSTD => CompressionType::ZSTD,
-        };
-        let mut compression_builder = ipc::Message::BodyCompressionBuilder::new(&mut fbb);
-        compression_builder.add_codec(compression);
-        Some(compression_builder.finish())
-    } else {
-        None
+    let message = arrow_format::ipc::Message {
+        version: arrow_format::ipc::MetadataVersion::V5,
+        header: Some(arrow_format::ipc::MessageHeader::RecordBatch(Box::new(
+            arrow_format::ipc::RecordBatch {
+                length: columns.len() as i64,
+                nodes: Some(nodes),
+                buffers: Some(buffers),
+                compression,
+            },
+        ))),
+        body_length: arrow_data.len() as i64,
+        custom_metadata: None,
     };
 
-    let root = {
-        let mut batch_builder = ipc::Message::RecordBatchBuilder::new(&mut fbb);
-        batch_builder.add_length(batch.num_rows() as i64);
-        batch_builder.add_nodes(nodes);
-        batch_builder.add_buffers(buffers);
-        if let Some(compression) = compression {
-            batch_builder.add_compression(compression)
-        }
-        let b = batch_builder.finish();
-        b.as_union_value()
-    };
-    // create an ipc::Schema::Message
-    let mut message = ipc::Message::MessageBuilder::new(&mut fbb);
-    message.add_version(ipc::Schema::MetadataVersion::V5);
-    message.add_header_type(ipc::Message::MessageHeader::RecordBatch);
-    message.add_bodyLength(arrow_data.len() as i64);
-    message.add_header(root);
-    let root = message.finish();
-    fbb.finish(root, None);
-    let finished_data = fbb.finished_data();
+    let mut builder = Builder::new();
+    let ipc_message = builder.finish(&message, None);
 
     EncodedData {
-        ipc_message: finished_data.to_vec(),
+        ipc_message: ipc_message.to_vec(),
         arrow_data,
     }
 }
@@ -285,10 +262,8 @@ fn dictionary_batch_to_bytes(
     options: &WriteOptions,
     is_little_endian: bool,
 ) -> EncodedData {
-    let mut fbb = FlatBufferBuilder::new();
-
-    let mut nodes: Vec<ipc::Message::FieldNode> = vec![];
-    let mut buffers: Vec<ipc::Schema::Buffer> = vec![];
+    let mut nodes: Vec<arrow_format::ipc::FieldNode> = vec![];
+    let mut buffers: Vec<arrow_format::ipc::Buffer> = vec![];
     let mut arrow_data: Vec<u8> = vec![];
 
     let length = write_dictionary(
@@ -302,54 +277,31 @@ fn dictionary_batch_to_bytes(
         false,
     );
 
-    // write data
-    let buffers = fbb.create_vector(&buffers);
-    let nodes = fbb.create_vector(&nodes);
+    let compression = serialize_compression(options.compression);
 
-    let compression = if let Some(compression) = options.compression {
-        let compression = match compression {
-            Compression::LZ4 => CompressionType::LZ4_FRAME,
-            Compression::ZSTD => CompressionType::ZSTD,
-        };
-        let mut compression_builder = ipc::Message::BodyCompressionBuilder::new(&mut fbb);
-        compression_builder.add_codec(compression);
-        Some(compression_builder.finish())
-    } else {
-        None
+    let message = arrow_format::ipc::Message {
+        version: arrow_format::ipc::MetadataVersion::V5,
+        header: Some(arrow_format::ipc::MessageHeader::DictionaryBatch(Box::new(
+            arrow_format::ipc::DictionaryBatch {
+                id: dict_id,
+                data: Some(Box::new(arrow_format::ipc::RecordBatch {
+                    length: length as i64,
+                    nodes: Some(nodes),
+                    buffers: Some(buffers),
+                    compression,
+                })),
+                is_delta: false,
+            },
+        ))),
+        body_length: arrow_data.len() as i64,
+        custom_metadata: None,
     };
 
-    let root = {
-        let mut batch_builder = ipc::Message::RecordBatchBuilder::new(&mut fbb);
-        batch_builder.add_length(length as i64);
-        batch_builder.add_nodes(nodes);
-        batch_builder.add_buffers(buffers);
-        if let Some(compression) = compression {
-            batch_builder.add_compression(compression)
-        }
-        batch_builder.finish()
-    };
-
-    let root = {
-        let mut batch_builder = ipc::Message::DictionaryBatchBuilder::new(&mut fbb);
-        batch_builder.add_id(dict_id);
-        batch_builder.add_data(root);
-        batch_builder.finish().as_union_value()
-    };
-
-    let root = {
-        let mut message_builder = ipc::Message::MessageBuilder::new(&mut fbb);
-        message_builder.add_version(ipc::Schema::MetadataVersion::V5);
-        message_builder.add_header_type(ipc::Message::MessageHeader::DictionaryBatch);
-        message_builder.add_bodyLength(arrow_data.len() as i64);
-        message_builder.add_header(root);
-        message_builder.finish()
-    };
-
-    fbb.finish(root, None);
-    let finished_data = fbb.finished_data();
+    let mut builder = Builder::new();
+    let ipc_message = builder.finish(&message, None);
 
     EncodedData {
-        ipc_message: finished_data.to_vec(),
+        ipc_message: ipc_message.to_vec(),
         arrow_data,
     }
 }
@@ -358,14 +310,14 @@ fn dictionary_batch_to_bytes(
 /// multiple times. Can optionally error if an update to an existing dictionary is attempted, which
 /// isn't allowed in the `FileWriter`.
 pub struct DictionaryTracker {
-    written: HashMap<i64, Arc<dyn Array>>,
+    written: Dictionaries,
     error_on_replacement: bool,
 }
 
 impl DictionaryTracker {
     pub fn new(error_on_replacement: bool) -> Self {
         Self {
-            written: HashMap::new(),
+            written: Dictionaries::new(),
             error_on_replacement,
         }
     }
@@ -381,7 +333,7 @@ impl DictionaryTracker {
     ///   inserted.
     pub fn insert(&mut self, dict_id: i64, array: &Arc<dyn Array>) -> Result<bool> {
         let values = match array.data_type() {
-            DataType::Dictionary(key_type, _) => {
+            DataType::Dictionary(key_type, _, _) => {
                 match_integer_type!(key_type, |$T| {
                     let array = array
                         .as_any()
@@ -414,6 +366,7 @@ impl DictionaryTracker {
 }
 
 /// Stores the encoded data, which is an ipc::Schema::Message, and optional Arrow data
+#[derive(Debug)]
 pub struct EncodedData {
     /// An encoded ipc::Schema::Message
     pub ipc_message: Vec<u8>,
