@@ -1,4 +1,4 @@
-use arrow_format::ipc::{Message, Schema};
+use arrow_format::ipc;
 
 use crate::{
     array::*,
@@ -14,7 +14,7 @@ use super::common::{pad_to_8, Compression};
 
 fn _write_primitive<T: NativeType>(
     array: &PrimitiveArray<T>,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
@@ -41,7 +41,7 @@ fn _write_primitive<T: NativeType>(
 
 fn write_primitive<T: NativeType>(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
@@ -60,7 +60,7 @@ fn write_primitive<T: NativeType>(
 
 fn write_boolean(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     _: bool,
@@ -91,7 +91,7 @@ fn write_generic_binary<O: Offset>(
     validity: Option<&Bitmap>,
     offsets: &[O],
     values: &[u8],
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
@@ -139,7 +139,7 @@ fn write_generic_binary<O: Offset>(
 
 fn write_binary<O: Offset>(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
@@ -160,7 +160,7 @@ fn write_binary<O: Offset>(
 
 fn write_utf8<O: Offset>(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
@@ -181,7 +181,7 @@ fn write_utf8<O: Offset>(
 
 fn write_fixed_size_binary(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     _is_little_endian: bool,
@@ -204,9 +204,9 @@ fn write_fixed_size_binary(
 
 fn write_list<O: Offset>(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
-    nodes: &mut Vec<Message::FieldNode>,
+    nodes: &mut Vec<ipc::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
     compression: Option<Compression>,
@@ -262,9 +262,9 @@ fn write_list<O: Offset>(
 
 pub fn write_struct(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
-    nodes: &mut Vec<Message::FieldNode>,
+    nodes: &mut Vec<ipc::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
     compression: Option<Compression>,
@@ -293,9 +293,9 @@ pub fn write_struct(
 
 pub fn write_union(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
-    nodes: &mut Vec<Message::FieldNode>,
+    nodes: &mut Vec<ipc::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
     compression: Option<Compression>,
@@ -336,9 +336,9 @@ pub fn write_union(
 
 fn write_map(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
-    nodes: &mut Vec<Message::FieldNode>,
+    nodes: &mut Vec<ipc::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
     compression: Option<Compression>,
@@ -394,9 +394,9 @@ fn write_map(
 
 fn write_fixed_size_list(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
-    nodes: &mut Vec<Message::FieldNode>,
+    nodes: &mut Vec<ipc::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
     compression: Option<Compression>,
@@ -425,9 +425,9 @@ fn write_fixed_size_list(
 #[allow(clippy::too_many_arguments)]
 pub fn _write_dictionary<K: DictionaryKey>(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
-    nodes: &mut Vec<Message::FieldNode>,
+    nodes: &mut Vec<ipc::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
     compression: Option<Compression>,
@@ -461,16 +461,16 @@ pub fn _write_dictionary<K: DictionaryKey>(
 #[allow(clippy::too_many_arguments)]
 pub fn write_dictionary(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
-    nodes: &mut Vec<Message::FieldNode>,
+    nodes: &mut Vec<ipc::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
     compression: Option<Compression>,
     write_keys: bool,
 ) -> usize {
     match array.data_type() {
-        DataType::Dictionary(key_type, _) => {
+        DataType::Dictionary(key_type, _, _) => {
             match_integer_type!(key_type, |$T| {
                 _write_dictionary::<$T>(
                     array,
@@ -490,17 +490,17 @@ pub fn write_dictionary(
 
 pub fn write(
     array: &dyn Array,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
-    nodes: &mut Vec<Message::FieldNode>,
+    nodes: &mut Vec<ipc::FieldNode>,
     offset: &mut i64,
     is_little_endian: bool,
     compression: Option<Compression>,
 ) {
-    nodes.push(Message::FieldNode::new(
-        array.len() as i64,
-        array.null_count() as i64,
-    ));
+    nodes.push(ipc::FieldNode {
+        length: array.len() as i64,
+        null_count: array.null_count() as i64,
+    });
     use PhysicalType::*;
     match array.data_type().to_physical_type() {
         Null => (),
@@ -637,7 +637,7 @@ fn pad_buffer_to_8(buffer: &mut Vec<u8>, length: usize) {
 /// writes `bytes` to `arrow_data` updating `buffers` and `offset` and guaranteeing a 8 byte boundary.
 fn write_bytes(
     bytes: &[u8],
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     compression: Option<Compression>,
@@ -660,14 +660,17 @@ fn write_bytes(
     pad_buffer_to_8(arrow_data, arrow_data.len() - start);
 
     let total_len = (arrow_data.len() - start) as i64;
-    buffers.push(Schema::Buffer::new(*offset, total_len));
+    buffers.push(ipc::Buffer {
+        offset: *offset,
+        length: total_len,
+    });
     *offset += total_len;
 }
 
 fn write_bitmap(
     bitmap: Option<&Bitmap>,
     length: usize,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     compression: Option<Compression>,
@@ -686,9 +689,10 @@ fn write_bitmap(
             }
         }
         None => {
-            // in IPC, the null bitmap is always be present
-            let slice = vec![0u8; length.saturating_add(7) / 8];
-            write_bytes(&slice, buffers, arrow_data, offset, compression)
+            buffers.push(ipc::Buffer {
+                offset: *offset,
+                length: 0,
+            });
         }
     }
 }
@@ -696,7 +700,7 @@ fn write_bitmap(
 /// writes `bytes` to `arrow_data` updating `buffers` and `offset` and guaranteeing a 8 byte boundary.
 fn write_buffer<T: NativeType>(
     buffer: &[T],
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
@@ -712,7 +716,10 @@ fn write_buffer<T: NativeType>(
     pad_buffer_to_8(arrow_data, arrow_data.len() - start);
 
     let total_len = (arrow_data.len() - start) as i64;
-    buffers.push(Schema::Buffer::new(*offset, total_len));
+    buffers.push(ipc::Buffer {
+        offset: *offset,
+        length: total_len,
+    });
     *offset += total_len;
 }
 
@@ -767,12 +774,7 @@ fn _write_compressed_buffer_from_iter<T: NativeType, I: TrustedLen<Item = T>>(
 fn _write_buffer<T: NativeType>(buffer: &[T], arrow_data: &mut Vec<u8>, is_little_endian: bool) {
     if is_little_endian == is_native_little_endian() {
         // in native endianess we can use the bytes directly.
-        let buffer = unsafe {
-            std::slice::from_raw_parts(
-                buffer.as_ptr() as *const u8,
-                buffer.len() * std::mem::size_of::<T>(),
-            )
-        };
+        let buffer = bytemuck::cast_slice(buffer);
         arrow_data.extend_from_slice(buffer);
     } else {
         _write_buffer_from_iter(buffer.iter().copied(), arrow_data, is_little_endian)
@@ -786,12 +788,7 @@ fn _write_compressed_buffer<T: NativeType>(
     compression: Compression,
 ) {
     if is_little_endian == is_native_little_endian() {
-        let bytes = unsafe {
-            std::slice::from_raw_parts(
-                buffer.as_ptr() as *const u8,
-                buffer.len() * std::mem::size_of::<T>(),
-            )
-        };
+        let bytes = bytemuck::cast_slice(buffer);
         arrow_data.extend_from_slice(&(bytes.len() as i64).to_le_bytes());
         match compression {
             Compression::LZ4 => {
@@ -810,7 +807,7 @@ fn _write_compressed_buffer<T: NativeType>(
 #[inline]
 fn write_buffer_from_iter<T: NativeType, I: TrustedLen<Item = T>>(
     buffer: I,
-    buffers: &mut Vec<Schema::Buffer>,
+    buffers: &mut Vec<ipc::Buffer>,
     arrow_data: &mut Vec<u8>,
     offset: &mut i64,
     is_little_endian: bool,
@@ -827,6 +824,9 @@ fn write_buffer_from_iter<T: NativeType, I: TrustedLen<Item = T>>(
     pad_buffer_to_8(arrow_data, arrow_data.len() - start);
 
     let total_len = (arrow_data.len() - start) as i64;
-    buffers.push(Schema::Buffer::new(*offset, total_len));
+    buffers.push(ipc::Buffer {
+        offset: *offset,
+        length: total_len,
+    });
     *offset += total_len;
 }
