@@ -1,11 +1,8 @@
 #![deny(missing_docs)]
 //! Convert data between the Arrow memory format and JSON line-delimited records.
 
-mod read;
-mod write;
-
-pub use read::*;
-pub use write::*;
+pub mod read;
+pub mod write;
 
 use crate::error::ArrowError;
 
