@@ -1,3 +1,4 @@
+#![forbid(unsafe_code)]
 //! Contains modules to interface with other formats such as [`csv`],
 //! [`parquet`], [`json`], [`ipc`], [`mod@print`] and [`avro`].
 #[cfg(any(
@@ -42,3 +43,6 @@ pub mod avro;
 #[cfg(feature = "io_print")]
 #[cfg_attr(docsrs, doc(cfg(feature = "io_print")))]
 pub mod print;
+
+#[cfg(any(feature = "io_csv_write", feature = "io_avro", feature = "io_json"))]
+mod iterator;
