@@ -12,7 +12,6 @@ use parquet2::{
     encoding::{hybrid_rle, Encoding},
     metadata::{ColumnChunkMetaData, ColumnDescriptor},
     page::DataPage,
-    FallibleStreamingIterator,
 };
 
 pub(super) fn read_required(buffer: &[u8], additional: usize, values: &mut MutableBitmap) {
@@ -68,25 +67,6 @@ fn read_optional(
     }
 }
 
-pub fn iter_to_array<I, E>(mut iter: I, metadata: &ColumnChunkMetaData) -> Result<BooleanArray>
-where
-    ArrowError: From<E>,
-    I: FallibleStreamingIterator<Item = DataPage, Error = E>,
-{
-    let capacity = metadata.num_values() as usize;
-    let mut values = MutableBitmap::with_capacity(capacity);
-    let mut validity = MutableBitmap::with_capacity(capacity);
-    while let Some(page) = iter.next()? {
-        extend_from_page(page, metadata.descriptor(), &mut values, &mut validity)?
-    }
-
-    Ok(BooleanArray::from_data(
-        DataType::Boolean,
-        values.into(),
-        validity.into(),
-    ))
-}
-
 pub async fn stream_to_array<I, E>(pages: I, metadata: &ColumnChunkMetaData) -> Result<BooleanArray>
 where
     ArrowError: From<E>,
@@ -115,7 +95,7 @@ where
     ))
 }
 
-fn extend_from_page(
+pub(super) fn extend_from_page(
     page: &DataPage,
     descriptor: &ColumnDescriptor,
     values: &mut MutableBitmap,
