@@ -9,7 +9,6 @@ use super::{ColumnChunkMetaData, ColumnDescriptor};
 use crate::{
     array::FixedSizeBinaryArray,
     bitmap::{utils::BitmapIter, MutableBitmap},
-    buffer::MutableBuffer,
     datatypes::DataType,
     error::{ArrowError, Result},
 };
@@ -24,7 +23,7 @@ pub(crate) fn read_dict_buffer(
     additional: usize,
     size: usize,
     dict: &FixedLenByteArrayPageDict,
-    values: &mut MutableBuffer<u8>,
+    values: &mut Vec<u8>,
     validity: &mut MutableBitmap,
 ) {
     let length = values.len() + additional * size;
@@ -50,7 +49,7 @@ pub(crate) fn read_dict_buffer(
                         let index = indices.next().unwrap() as usize;
                         values.extend_from_slice(&dict_values[index * size..(index + 1) * size]);
                     } else {
-                        values.extend_constant(size, 0);
+                        values.resize(values.len() + size, 0);
                     }
                 }
             }
@@ -63,19 +62,44 @@ pub(crate) fn read_dict_buffer(
                         values.extend_from_slice(&dict_values[index * size..(index + 1) * size]);
                     })
                 } else {
-                    values.extend_constant(additional * size, 0)
+                    values.resize(values.len() + additional * size, 0);
                 }
             }
         }
     }
 }
 
+/// Assumptions: No rep levels
+pub(crate) fn read_dict_required(
+    indices_buffer: &[u8],
+    additional: usize,
+    size: usize,
+    dict: &FixedLenByteArrayPageDict,
+    values: &mut Vec<u8>,
+    validity: &mut MutableBitmap,
+) {
+    let dict_values = dict.values();
+
+    // SPEC: Data page format: the bit width used to encode the entry ids stored as 1 byte (max bit width = 32),
+    // SPEC: followed by the values encoded using RLE/Bit packed described above (with the given bit width).
+    let bit_width = indices_buffer[0];
+    let indices_buffer = &indices_buffer[1..];
+
+    let indices = hybrid_rle::HybridRleDecoder::new(indices_buffer, bit_width as u32, additional);
+
+    for index in indices {
+        let index = index as usize;
+        values.extend_from_slice(&dict_values[index * size..(index + 1) * size]);
+    }
+    validity.extend_constant(additional * size, true);
+}
+
 pub(crate) fn read_optional(
     validity_buffer: &[u8],
     values_buffer: &[u8],
     additional: usize,
     size: usize,
-    values: &mut MutableBuffer<u8>,
+    values: &mut Vec<u8>,
     validity: &mut MutableBitmap,
 ) {
     let length = values.len() + additional * size;
@@ -97,7 +121,7 @@ pub(crate) fn read_optional(
                         let value = values_iterator.next().unwrap();
                         values.extend_from_slice(value);
                     } else {
-                        values.extend_constant(size, 0)
+                        values.resize(values.len() + size, 0);
                     }
                 }
             }
@@ -110,19 +134,14 @@ pub(crate) fn read_optional(
                         values.extend_from_slice(value)
                     })
                 } else {
-                    values.extend_constant(additional * size, 0)
+                    values.resize(values.len() + additional * size, 0);
                 }
             }
         }
     }
 }
 
-pub(crate) fn read_required(
-    buffer: &[u8],
-    additional: usize,
-    size: usize,
-    values: &mut MutableBuffer<u8>,
-) {
+pub(crate) fn read_required(buffer: &[u8], additional: usize, size: usize, values: &mut Vec<u8>) {
     assert_eq!(buffer.len(), additional * size);
     values.extend_from_slice(buffer);
 }
@@ -139,7 +158,7 @@ where
     let size = FixedSizeBinaryArray::get_size(&data_type);
 
     let capacity = metadata.num_values() as usize;
-    let mut values = MutableBuffer::<u8>::with_capacity(capacity * size);
+    let mut values = Vec::<u8>::with_capacity(capacity * size);
     let mut validity = MutableBitmap::with_capacity(capacity);
     while let Some(page) = iter.next()? {
         extend_from_page(
@@ -171,7 +190,7 @@ where
     let size = FixedSizeBinaryArray::get_size(&data_type);
 
     let capacity = metadata.num_values() as usize;
-    let mut values = MutableBuffer::<u8>::with_capacity(capacity * size);
+    let mut values = Vec::<u8>::with_capacity(capacity * size);
     let mut validity = MutableBitmap::with_capacity(capacity);
 
     pin_mut!(pages); // needed for iteration
@@ -197,7 +216,7 @@ pub(crate) fn extend_from_page(
     page: &DataPage,
     size: usize,
     descriptor: &ColumnDescriptor,
-    values: &mut MutableBuffer<u8>,
+    values: &mut Vec<u8>,
     validity: &mut MutableBitmap,
 ) -> Result<()> {
     let additional = page.num_values();
@@ -217,6 +236,14 @@ pub(crate) fn extend_from_page(
             values,
             validity,
         ),
+        (Encoding::PlainDictionary, Some(dict), false) => read_dict_required(
+            values_buffer,
+            additional,
+            size,
+            dict.as_any().downcast_ref().unwrap(),
+            values,
+            validity,
+        ),
         (Encoding::Plain, _, true) => read_optional(
             validity_buffer,
             values_buffer,
