@@ -3,7 +3,6 @@
 
 use std::{
     collections::VecDeque,
-    convert::TryInto,
     io::{Read, Seek},
     sync::Arc,
 };
@@ -98,7 +97,7 @@ fn dict_read<
     data_type: DataType,
 ) -> Result<Box<dyn Array>> {
     use DataType::*;
-    let values_data_type = if let Dictionary(_, v) = &data_type {
+    let values_data_type = if let Dictionary(_, v, _) = &data_type {
         v.as_ref()
     } else {
         panic!()
@@ -144,13 +143,41 @@ fn dict_read<
             )
         }
         Timestamp(TimeUnit::Nanosecond, None) => match metadata.descriptor().type_() {
-            ParquetType::PrimitiveType { physical_type, .. } => match physical_type {
-                PhysicalType::Int96 => primitive::iter_to_dict_array::<K, _, _, _, _, _>(
+            ParquetType::PrimitiveType {
+                physical_type,
+                logical_type,
+                ..
+            } => match (physical_type, logical_type) {
+                (PhysicalType::Int96, _) => primitive::iter_to_dict_array::<K, _, _, _, _, _>(
                     iter,
                     metadata,
                     DataType::Timestamp(TimeUnit::Nanosecond, None),
                     int96_to_i64_ns,
                 ),
+                (_, Some(LogicalType::TIMESTAMP(TimestampType { unit, .. }))) => match unit {
+                    ParquetTimeUnit::MILLIS(_) => {
+                        primitive::iter_to_dict_array::<K, _, _, _, _, _>(
+                            iter,
+                            metadata,
+                            data_type,
+                            |x: i64| x * 1_000_000,
+                        )
+                    }
+                    ParquetTimeUnit::MICROS(_) => {
+                        primitive::iter_to_dict_array::<K, _, _, _, _, _>(
+                            iter,
+                            metadata,
+                            data_type,
+                            |x: i64| x * 1_000,
+                        )
+                    }
+                    ParquetTimeUnit::NANOS(_) => primitive::iter_to_dict_array::<K, _, _, _, _, _>(
+                        iter,
+                        metadata,
+                        data_type,
+                        |x: i64| x,
+                    ),
+                },
                 _ => primitive::iter_to_dict_array::<K, _, _, _, _, _>(
                     iter,
                     metadata,
@@ -196,17 +223,17 @@ fn column_datatype(data_type: &DataType, column: usize) -> DataType {
         | LargeUtf8 | Dictionary(_) | List | LargeList | FixedSizeList => data_type.clone(),
         Struct => {
             if let DataType::Struct(fields) = data_type.to_logical_type() {
-                let mut total_columns = 0;
+                let mut total_chunk = 0;
                 let mut total_fields = 0;
                 for f in fields {
-                    let field_columns = column_offset(f.data_type());
-                    if column < total_columns + field_columns {
-                        return column_datatype(f.data_type(), column + total_columns);
+                    let field_chunk = column_offset(f.data_type());
+                    if column < total_chunk + field_chunk {
+                        return column_datatype(f.data_type(), column + total_chunk);
                     }
-                    total_fields += (field_columns > 0) as usize;
-                    total_columns += field_columns;
+                    total_fields += (field_chunk > 0) as usize;
+                    total_chunk += field_chunk;
                 }
-                fields[column + total_fields - total_columns]
+                fields[column + total_fields - total_chunk]
                     .data_type()
                     .clone()
             } else {
@@ -243,14 +270,33 @@ fn page_iter_to_array<I: FallibleStreamingIterator<Item = DataPage, Error = Parq
         }
 
         Timestamp(TimeUnit::Nanosecond, None) => match metadata.descriptor().type_() {
-            ParquetType::PrimitiveType { physical_type, .. } => match physical_type {
-                PhysicalType::Int96 => primitive::iter_to_array(
+            ParquetType::PrimitiveType {
+                physical_type,
+                logical_type,
+                ..
+            } => match (physical_type, logical_type) {
+                (PhysicalType::Int96, _) => primitive::iter_to_array(
                     iter,
                     metadata,
                     DataType::Timestamp(TimeUnit::Nanosecond, None),
                     nested,
                     int96_to_i64_ns,
                 ),
+                (_, Some(LogicalType::TIMESTAMP(TimestampType { unit, .. }))) => match unit {
+                    ParquetTimeUnit::MILLIS(_) => {
+                        primitive::iter_to_array(iter, metadata, data_type, nested, |x: i64| {
+                            x * 1_000_000
+                        })
+                    }
+                    ParquetTimeUnit::MICROS(_) => {
+                        primitive::iter_to_array(iter, metadata, data_type, nested, |x: i64| {
+                            x * 1_000
+                        })
+                    }
+                    ParquetTimeUnit::NANOS(_) => {
+                        primitive::iter_to_array(iter, metadata, data_type, nested, |x: i64| x)
+                    }
+                },
                 _ => primitive::iter_to_array(iter, metadata, data_type, nested, |x: i64| x),
             },
             _ => unreachable!(),
@@ -267,36 +313,36 @@ fn page_iter_to_array<I: FallibleStreamingIterator<Item = DataPage, Error = Parq
                 PhysicalType::Int64 => {
                     primitive::iter_to_array(iter, metadata, data_type, nested, |x: i64| x as i128)
                 }
-                PhysicalType::FixedLenByteArray(n) => {
-                    if *n > 16 {
-                        Err(ArrowError::NotYetImplemented(format!(
-                            "Can't decode Decimal128 type from Fixed Size Byte Array of len {:?}",
-                            n
-                        )))
-                    } else {
-                        let paddings = (0..(16 - *n)).map(|_| 0u8).collect::<Vec<_>>();
-                        fixed_size_binary::iter_to_array(
-                            iter,
-                            DataType::FixedSizeBinary(*n as usize),
-                            metadata,
-                        )
-                        .map(|e| {
-                            let a = e
-                                .into_iter()
-                                .map(|v| {
-                                    v.and_then(|v1| {
-                                        [&paddings, v1]
-                                            .concat()
-                                            .try_into()
-                                            .map(i128::from_be_bytes)
-                                            .ok()
-                                    })
-                                })
-                                .collect::<Vec<_>>();
-                            Box::new(PrimitiveArray::<i128>::from(a).to(data_type))
-                                as Box<dyn Array>
+                &PhysicalType::FixedLenByteArray(n) if n > 16 => {
+                    Err(ArrowError::NotYetImplemented(format!(
+                        "Can't decode Decimal128 type from Fixed Size Byte Array of len {:?}",
+                        n
+                    )))
+                }
+                &PhysicalType::FixedLenByteArray(n) => {
+                    let n = n as usize;
+                    let fixed_size_binary_array = fixed_size_binary::iter_to_array(
+                        iter,
+                        DataType::FixedSizeBinary(n),
+                        metadata,
+                    )?;
+                    let values = fixed_size_binary_array
+                        .values()
+                        .chunks_exact(n)
+                        .map(|value: &[u8]| {
+                            // Copy the fixed-size byte value to the start of a 16 byte stack
+                            // allocated buffer, then use an arithmetic right shift to fill in
+                            // MSBs, which accounts for leading 1's in negative (two's complement)
+                            // values.
+                            let mut bytes = [0u8; 16];
+                            bytes[..n].copy_from_slice(value);
+                            i128::from_be_bytes(bytes) >> (8 * (16 - n))
                         })
-                    }
+                        .collect::<Vec<_>>();
+                    let validity = fixed_size_binary_array.validity().cloned();
+                    let i128_array =
+                        PrimitiveArray::<i128>::from_data(data_type, values.into(), validity);
+                    Ok(Box::new(i128_array) as _)
                 }
                 _ => unreachable!(),
             },
@@ -317,7 +363,7 @@ fn page_iter_to_array<I: FallibleStreamingIterator<Item = DataPage, Error = Parq
             binary::iter_to_array::<i64, _, _>(iter, metadata, data_type, nested)
         }
 
-        Dictionary(key_type, _) => match_integer_type!(key_type, |$T| {
+        Dictionary(key_type, _, _) => match_integer_type!(key_type, |$T| {
             dict_read::<$T, _>(iter, metadata, data_type)
         }),
 
