@@ -4,9 +4,10 @@ use std::{
 };
 
 use crate::{
+    array::Array,
+    chunk::Chunk,
     datatypes::{Field, Schema},
     error::{ArrowError, Result},
-    record_batch::RecordBatch,
 };
 
 use super::{
@@ -16,7 +17,7 @@ use super::{
 
 type GroupFilter = Arc<dyn Fn(usize, &RowGroupMetaData) -> bool>;
 
-/// Single threaded iterator of [`RecordBatch`] from a parquet file.
+/// Single threaded iterator of a paquet file.
 pub struct RecordReader<R: Read + Seek> {
     reader: R,
     schema: Arc<Schema>,
@@ -107,10 +108,10 @@ impl<R: Read + Seek> RecordReader<R> {
 }
 
 impl<R: Read + Seek> Iterator for RecordReader<R> {
-    type Item = Result<RecordBatch>;
+    type Item = Result<Chunk<Arc<dyn Array>>>;
 
     fn next(&mut self) -> Option<Self::Item> {
-        if self.schema.fields().is_empty() {
+        if self.schema.fields.is_empty() {
             return None;
         }
         if self.current_group == self.metadata.row_groups.len() {
@@ -137,8 +138,8 @@ impl<R: Read + Seek> Iterator for RecordReader<R> {
         let b1 = std::mem::take(&mut self.buffer);
         let b2 = std::mem::take(&mut self.decompress_buffer);
 
-        let a = schema.fields().iter().enumerate().try_fold(
-            (b1, b2, Vec::with_capacity(schema.fields().len())),
+        let a = schema.fields.iter().enumerate().try_fold(
+            (b1, b2, Vec::with_capacity(schema.fields.len())),
             |(b1, b2, mut columns), (field_index, field)| {
                 let field_index = self.indices[field_index]; // project into the original schema
                 let column_iter = get_column_iterator(
@@ -158,19 +159,18 @@ impl<R: Read + Seek> Iterator for RecordReader<R> {
                     array
                 };
 
-                columns.push(array.into());
+                let column: Arc<dyn Array> = array.into();
+                columns.push(column);
                 Result::Ok((b1, b2, columns))
             },
         );
 
         self.current_group += 1;
-        Some(a.and_then(|(b1, b2, columns)| {
+        Some(a.map(|(b1, b2, columns)| {
             self.buffer = b1;
             self.decompress_buffer = b2;
-            RecordBatch::try_new(self.schema.clone(), columns).map(|batch| {
-                self.remaining_rows -= batch.num_rows();
-                batch
-            })
+            self.remaining_rows -= columns[0].len();
+            Chunk::new(columns)
         }))
     }
 }
