@@ -6,30 +6,31 @@ use super::{
     SchemaDescriptor, WriteOptions,
 };
 use crate::{
+    array::Array,
+    chunk::Chunk,
     datatypes::Schema,
     error::{ArrowError, Result},
-    record_batch::RecordBatch,
 };
 
-/// An iterator adapter that converts an iterator over [`RecordBatch`] into an iterator
+/// An iterator adapter that converts an iterator over [`Chunk`] into an iterator
 /// of row groups.
 /// Use it to create an iterator consumable by the parquet's API.
-pub struct RowGroupIterator<I: Iterator<Item = Result<RecordBatch>>> {
+pub struct RowGroupIterator<A: AsRef<dyn Array> + 'static, I: Iterator<Item = Result<Chunk<A>>>> {
     iter: I,
     options: WriteOptions,
     parquet_schema: SchemaDescriptor,
     encodings: Vec<Encoding>,
 }
 
-impl<'a, I: Iterator<Item = Result<RecordBatch>>> RowGroupIterator<I> {
-    /// Creates a new [`RowGroupIterator`] from an iterator over [`RecordBatch`].
+impl<A: AsRef<dyn Array> + 'static, I: Iterator<Item = Result<Chunk<A>>>> RowGroupIterator<A, I> {
+    /// Creates a new [`RowGroupIterator`] from an iterator over [`Chunk`].
     pub fn try_new(
         iter: I,
         schema: &Schema,
         options: WriteOptions,
         encodings: Vec<Encoding>,
     ) -> Result<Self> {
-        assert_eq!(schema.fields().len(), encodings.len());
+        assert_eq!(schema.fields.len(), encodings.len());
 
         let parquet_schema = to_parquet_schema(schema)?;
 
@@ -47,18 +48,20 @@ impl<'a, I: Iterator<Item = Result<RecordBatch>>> RowGroupIterator<I> {
     }
 }
 
-impl<I: Iterator<Item = Result<RecordBatch>>> Iterator for RowGroupIterator<I> {
+impl<A: AsRef<dyn Array> + 'static + Send + Sync, I: Iterator<Item = Result<Chunk<A>>>> Iterator
+    for RowGroupIterator<A, I>
+{
     type Item = Result<RowGroupIter<'static, ArrowError>>;
 
     fn next(&mut self) -> Option<Self::Item> {
         let options = self.options;
 
-        self.iter.next().map(|batch| {
-            let batch = batch?;
-            let columns = batch.columns().to_vec();
+        self.iter.next().map(|maybe_chunk| {
+            let columns = maybe_chunk?;
             let encodings = self.encodings.clone();
             Ok(DynIter::new(
                 columns
+                    .into_arrays()
                     .into_iter()
                     .zip(self.parquet_schema.columns().to_vec().into_iter())
                     .zip(encodings.into_iter())
