@@ -13,5 +13,9 @@ mod parquet;
 #[cfg(feature = "io_avro")]
 mod avro;
 
-#[cfg(feature = "io_csv")]
+#[cfg(any(
+    feature = "io_csv_read",
+    feature = "io_csv_write",
+    feature = "io_csv_read_async"
+))]
 mod csv;
