@@ -3,9 +3,8 @@ use std::sync::Arc;
 
 use arrow2::error::ArrowError;
 use arrow2::{
-    array::*, bitmap::Bitmap, buffer::Buffer, datatypes::*, error::Result,
+    array::*, bitmap::Bitmap, buffer::Buffer, chunk::Chunk, datatypes::*, error::Result,
     io::parquet::read::statistics::*, io::parquet::read::*, io::parquet::write::*,
-    record_batch::RecordBatch,
 };
 
 use crate::io::ipc::read_gzip_json;
@@ -34,7 +33,7 @@ pub fn read_column<R: Read + Seek>(
 }
 
 pub fn pyarrow_nested_nullable(column: usize) -> Box<dyn Array> {
-    let offsets = Buffer::<i32>::from([0, 2, 2, 5, 8, 8, 11, 11, 12]);
+    let offsets = Buffer::from_slice([0, 2, 2, 5, 8, 8, 11, 11, 12]);
 
     let values = match column {
         0 => {
@@ -321,6 +320,10 @@ pub fn pyarrow_nullable(column: usize) -> Box<dyn Array> {
                 .collect::<Vec<_>>();
             Box::new(PrimitiveArray::<i128>::from(values).to(DataType::Decimal(26, 0)))
         }
+        10 => Box::new(
+            PrimitiveArray::<i64>::from(i64_values)
+                .to(DataType::Timestamp(TimeUnit::Microsecond, None)),
+        ),
         _ => unreachable!(),
     }
 }
@@ -392,6 +395,13 @@ pub fn pyarrow_nullable_statistics(column: usize) -> Option<Box<dyn Statistics>>
             max_value: Some(9i128),
             data_type: DataType::Decimal(26, 0),
         }),
+        10 => Box::new(PrimitiveStatistics::<i64> {
+            data_type: DataType::Timestamp(TimeUnit::Microsecond, None),
+            distinct_count: None,
+            null_count: Some(3),
+            min_value: Some(0),
+            max_value: Some(9),
+        }),
         _ => unreachable!(),
     })
 }
@@ -399,6 +409,8 @@ pub fn pyarrow_nullable_statistics(column: usize) -> Option<Box<dyn Statistics>>
 // these values match the values in `integration`
 pub fn pyarrow_required(column: usize) -> Box<dyn Array> {
     let i64_values = &[
+        Some(-256),
+        Some(-1),
         Some(0),
         Some(1),
         Some(2),
@@ -407,8 +419,6 @@ pub fn pyarrow_required(column: usize) -> Box<dyn Array> {
         Some(5),
         Some(6),
         Some(7),
-        Some(8),
-        Some(9),
     ];
 
     match column {
@@ -610,7 +620,7 @@ pub fn pyarrow_struct_statistics(column: usize) -> Option<Box<dyn Statistics>> {
 }
 
 /// Round-trip with parquet using the same integration files used for IPC integration tests.
-fn integration_write(schema: &Schema, batches: &[RecordBatch]) -> Result<Vec<u8>> {
+fn integration_write(schema: &Schema, batches: &[Chunk<Arc<dyn Array>>]) -> Result<Vec<u8>> {
     let options = WriteOptions {
         write_statistics: true,
         compression: Compression::Uncompressed,
@@ -626,7 +636,7 @@ fn integration_write(schema: &Schema, batches: &[RecordBatch]) -> Result<Vec<u8>
             .iter()
             .zip(descritors.clone())
             .map(|(array, descriptor)| {
-                let encoding = if let DataType::Dictionary(_, _) = array.data_type() {
+                let encoding = if let DataType::Dictionary(..) = array.data_type() {
                     Encoding::RleDictionary
                 } else {
                     Encoding::Plain
@@ -657,7 +667,9 @@ fn integration_write(schema: &Schema, batches: &[RecordBatch]) -> Result<Vec<u8>
     Ok(writer.into_inner())
 }
 
-fn integration_read(data: &[u8]) -> Result<(Arc<Schema>, Vec<RecordBatch>)> {
+type IntegrationRead = (Arc<Schema>, Vec<Chunk<Arc<dyn Array>>>);
+
+fn integration_read(data: &[u8]) -> Result<IntegrationRead> {
     let reader = Cursor::new(data);
     let reader = RecordReader::try_new(reader, None, None, None, None)?;
     let schema = reader.schema().clone();
@@ -667,7 +679,7 @@ fn integration_read(data: &[u8]) -> Result<(Arc<Schema>, Vec<RecordBatch>)> {
 }
 
 fn test_file(version: &str, file_name: &str) -> Result<()> {
-    let (schema, batches) = read_gzip_json(version, file_name)?;
+    let (schema, _, batches) = read_gzip_json(version, file_name)?;
 
     let data = integration_write(&schema, &batches)?;
 
@@ -704,14 +716,11 @@ fn arrow_type() -> Result<()> {
     let dt1 = DataType::Duration(TimeUnit::Second);
     let array = PrimitiveArray::<i64>::from([Some(1), None, Some(2)]).to(dt1.clone());
     let array2 = Utf8Array::<i64>::from([Some("a"), None, Some("bb")]);
-    let schema = Schema::new(vec![
+    let schema = Schema::from(vec![
         Field::new("a1", dt1, true),
         Field::new("a2", array2.data_type().clone(), true),
     ]);
-    let batch = RecordBatch::try_new(
-        Arc::new(schema.clone()),
-        vec![Arc::new(array), Arc::new(array2)],
-    )?;
+    let batch = Chunk::try_new(vec![Arc::new(array) as Arc<dyn Array>, Arc::new(array2)])?;
 
     let r = integration_write(&schema, &[batch.clone()])?;
 
